/*
 *   -- clMAGMA (version 1.3.1) --
 *      Univ. of Tennessee, Knoxville
 *      Univ. of California, Berkeley
 *      Univ. of Colorado, Denver
 *      @date January 2015
 *
 * @author Mark Gates
 * @precisions normal z -> s d c
 */

#ifndef MAGMA_Z_H
#define MAGMA_Z_H

#include "magma_types.h"
// To Do - fix circular dependency for MAGMA MIC sparse 
// #include "../sparse-iter/include/magmasparse_types.h"

#define PRECISION_z

#ifdef __cplusplus
extern "C" {
#endif

/* ////////////////////////////////////////////////////////////////////////////
   -- MAGMA Auxiliary functions to get the NB used
*/
magma_int_t magma_get_dlaex3_m_nb();       // defined in dlaex3_m.cpp

magma_int_t magma_get_zpotrf_nb( magma_int_t m );
magma_int_t magma_get_zgetrf_nb( magma_int_t m );
magma_int_t magma_get_zgetri_nb( magma_int_t m );
magma_int_t magma_get_zgeqp3_nb( magma_int_t m );
magma_int_t magma_get_zgeqrf_nb( magma_int_t m );
magma_int_t magma_get_zgeqlf_nb( magma_int_t m );
magma_int_t magma_get_zgehrd_nb( magma_int_t m );
magma_int_t magma_get_zhetrd_nb( magma_int_t m );
magma_int_t magma_get_zhetrf_nb( magma_int_t m );
magma_int_t magma_get_zhetrf_nopiv_nb( magma_int_t m );
magma_int_t magma_get_zgelqf_nb( magma_int_t m );
magma_int_t magma_get_zgebrd_nb( magma_int_t m );
magma_int_t magma_get_zhegst_nb( magma_int_t m );
magma_int_t magma_get_zgesvd_nb( magma_int_t m );
magma_int_t magma_get_zhegst_nb_m( magma_int_t m );
magma_int_t magma_get_zbulge_nb( magma_int_t m, magma_int_t nbthreads );
magma_int_t magma_get_zbulge_nb_mgpu( magma_int_t m );
magma_int_t magma_zbulge_get_Vblksiz( magma_int_t m, magma_int_t nb, magma_int_t nbthreads );
magma_int_t magma_get_zbulge_gcperf();

/* ////////////////////////////////////////////////////////////////////
 magma symv for cpu
 */

void magmablas_zhemv_work_cpu(magma_uplo_t uplo, magma_int_t *pN, magmaDoubleComplex *palpha, magmaDoubleComplex *A, magma_int_t *plda, magmaDoubleComplex *X, magma_int_t *pincx, magmaDoubleComplex *pbeta, magmaDoubleComplex *Y, magma_int_t *pincy, magmaDoubleComplex *y);

void magmablas_zhemv_cpu(magma_uplo_t uplo, magma_int_t *pN, magmaDoubleComplex *palpha, magmaDoubleComplex *A, magma_int_t *plda, magmaDoubleComplex *X, magma_int_t *pincx, magmaDoubleComplex *pbeta, magmaDoubleComplex *Y, magma_int_t *pincy);

/* ////////////////////////////////////////////////////////////////////////////
   -- MAGMA function definitions / Data on CPU
*/
magma_int_t
magma_zgetrf(magma_int_t m, magma_int_t n,
             magmaDoubleComplex *a, magma_int_t lda,
             magma_int_t *ipiv, magma_int_t *info,
             magma_queue_t queue );

magma_int_t
magma_zgetf2_nopiv(magma_int_t m, magma_int_t n,
             magmaDoubleComplex *A, magma_int_t lda,
             magma_int_t *info);

magma_int_t
magma_zgetrf_nopiv(magma_int_t m, magma_int_t n,
    magmaDoubleComplex *A, magma_int_t lda,
    magma_int_t *info);

magma_int_t
magma_zgetrf3(magma_int_t m, magma_int_t n,
             magmaDoubleComplex *a, magma_int_t lda, magmaDoubleComplex *da, magma_int_t ldda,
             magma_int_t *ipiv, magma_int_t *d_ipiv, magma_int_t *info,
             magma_queue_t queue );


magma_int_t
magma_zpotrf(
        magma_uplo_t uplo,
        magma_int_t n,
        magmaDoubleComplex_ptr hA, magma_int_t lda,
        magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_zpotrf3(
        magma_uplo_t uplo,
        magma_int_t n,
        magmaDoubleComplex_ptr hA, magma_int_t lda, magmaDoubleComplex_ptr dA, magma_int_t ldda,
        magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_zgeqrf(magma_int_t m, magma_int_t n,
             magmaDoubleComplex *hA, magma_int_t lda,
             magmaDoubleComplex *tau, magmaDoubleComplex *hwork,
             magma_int_t lwork, magma_int_t *info,
             magma_queue_t queue );

magma_int_t
magma_zgeqrf3(magma_int_t m, magma_int_t n,
             magmaDoubleComplex *hA, magma_int_t lda, magmaDoubleComplex *dA, magma_int_t ldda,
             magmaDoubleComplex *tau, magmaDoubleComplex *hwork,
             magma_int_t lwork, magma_int_t *info,
             magma_queue_t queue );

magma_int_t
magma_zgesdd(magma_vec_t jobz, magma_int_t m, magma_int_t n,
             magmaDoubleComplex *A, magma_int_t lda,
             double *s,
             magmaDoubleComplex *U, magma_int_t ldu,
             magmaDoubleComplex *VT, magma_int_t ldvt,
             magmaDoubleComplex *work, magma_int_t lwork,
             #if defined(PRECISION_z) || defined(PRECISION_c)
             double *rwork,
             #endif
             magma_int_t *iwork,
             magma_queue_t queue, magma_int_t *info);

magma_int_t
magma_zgetrf2(magma_int_t m, magma_int_t n,
              magmaDoubleComplex *a, magma_int_t lda,
              magma_int_t *ipiv,
              magmaDoubleComplex_ptr dA, magma_int_t ldda, magma_int_t *d_ipiv,
              magma_int_t *info, magma_queue_t queue );

magma_int_t
magma_zunmtr( magma_side_t side, magma_uplo_t uplo, magma_trans_t trans,
              magma_int_t m, magma_int_t n,
              magmaDoubleComplex *a,    magma_int_t lda,
              magmaDoubleComplex *tau,
              magmaDoubleComplex *c,    magma_int_t ldc,
              magmaDoubleComplex *work, magma_int_t lwork,
              magma_int_t *info, magma_queue_t queue );

#if defined(PRECISION_z) || defined(PRECISION_c)
magma_int_t
magma_zgeev(  magma_vec_t jobvl, magma_vec_t jobvr, magma_int_t n,
              magmaDoubleComplex *a, magma_int_t lda,
              magmaDoubleComplex *w,
              magmaDoubleComplex *vl, magma_int_t ldvl,
              magmaDoubleComplex *vr, magma_int_t ldvr,
              magmaDoubleComplex *work, magma_int_t lwork,
              double *rwork, magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_zgesvd( magma_vec_t jobu, magma_vec_t jobvt, magma_int_t m, magma_int_t n,
              magmaDoubleComplex *a,    magma_int_t lda, double *s,
              magmaDoubleComplex *u,    magma_int_t ldu,
              magmaDoubleComplex *vt,   magma_int_t ldvt,
              magmaDoubleComplex *work, magma_int_t lwork,
              double *rwork, magma_int_t *info, magma_queue_t queue );

magma_int_t
magma_zheevd( magma_vec_t jobz, magma_uplo_t uplo, magma_int_t n,
              magmaDoubleComplex *a, magma_int_t lda, double *w,
              magmaDoubleComplex *work, magma_int_t lwork,
              double *rwork, magma_int_t lrwork,
              magma_int_t *iwork, magma_int_t liwork, magma_int_t *info,
              magma_queue_t queue );

magma_int_t
magma_zstedx( magma_range_t range, magma_int_t n, double vl, double vu,
              magma_int_t il, magma_int_t iu, double *D, double *E,
              magmaDoubleComplex *Z, magma_int_t ldz,
              double *rwork, magma_int_t ldrwork, magma_int_t *iwork,
              magma_int_t liwork, double *dwork, magma_int_t *info,
              magma_queue_t queue );

#else /* not (defined(PRECISION_z) || defined(PRECISION_c)) */

magma_int_t
magma_zgeev(  magma_vec_t jobvl, magma_vec_t jobvr, magma_int_t n,
              magmaDoubleComplex *a,    magma_int_t lda,
              magmaDoubleComplex *wr, magmaDoubleComplex *wi,
              magmaDoubleComplex *vl,   magma_int_t ldvl,
              magmaDoubleComplex *vr,   magma_int_t ldvr,
              magmaDoubleComplex *work, magma_int_t lwork,
              magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_zgesvd( magma_vec_t jobu, magma_vec_t jobvt, magma_int_t m, magma_int_t n,
              magmaDoubleComplex *a,    magma_int_t lda, double *s,
              magmaDoubleComplex *u,    magma_int_t ldu,
              magmaDoubleComplex *vt,   magma_int_t ldvt,
              magmaDoubleComplex *work, magma_int_t lwork,
              magma_int_t *info, magma_queue_t queue );

magma_int_t
magma_zheevd( magma_vec_t jobz, magma_uplo_t uplo, magma_int_t n,
              magmaDoubleComplex *a, magma_int_t lda, double *w,
              magmaDoubleComplex *work, magma_int_t lwork,
              magma_int_t *iwork, magma_int_t liwork, magma_int_t *info,
              magma_queue_t queue);

magma_int_t
magma_zstedx( magma_range_t range, magma_int_t n, double vl, double vu,
              magma_int_t il, magma_int_t iu, double *d, double *e,
              double *z, magma_int_t ldz,
              double *work, magma_int_t lwork,
              magma_int_t *iwork, magma_int_t liwork,
              double *dwork, magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_zlaex0( magma_int_t n, double *d, double *e, double *q, magma_int_t ldq,
              double *work, magma_int_t *iwork, double *dwork,
              magma_range_t range, double vl, double vu,
              magma_int_t il, magma_int_t iu, magma_int_t *info,
              magma_queue_t queue);

magma_int_t
magma_zlaex1( magma_int_t n, double *d, double *q, magma_int_t ldq,
              magma_int_t *indxq, double rho, magma_int_t cutpnt,
              double *work, magma_int_t *iwork, double *dwork,
              magma_range_t range, double vl, double vu,
              magma_int_t il, magma_int_t iu, magma_int_t *info,
              magma_queue_t queue);

magma_int_t
magma_zlaex3( magma_int_t k, magma_int_t n, magma_int_t n1, double *d,
              double *q, magma_int_t ldq, double rho,
              double *dlamda, double *q2, magma_int_t *indx,
              magma_int_t *ctot, double *w, double *s, magma_int_t *indxq,
              double *dwork,
              magma_range_t range, double vl, double vu, magma_int_t il, magma_int_t iu,
              magma_int_t *info, magma_queue_t queue );

#endif /* not (defined(PRECISION_z) || defined(PRECISION_c)) */


/* ////////////////////////////////////////////////////////////////////////////
 -- MAGMA function definitions / Data on MIC
*/

magma_int_t
magma_zgebrd(magma_int_t m, magma_int_t n,
             magmaDoubleComplex *a, magma_int_t lda, double *d, double *e,
             magmaDoubleComplex *tauq, magmaDoubleComplex *taup,
             magmaDoubleComplex *work, magma_int_t lwork,
             magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_zgeqrf2_mic(
        magma_int_t m, magma_int_t n,
        magmaDoubleComplex_ptr dA, size_t dA_offset, magma_int_t ldda,
        magmaDoubleComplex *tau, magma_int_t *info, magma_queue_t queue);

magma_int_t 
magma_zgerbt_mic(magma_bool_t gen, magma_int_t n, magma_int_t nrhs, 
        magmaDoubleComplex *dA, magma_int_t ldda, 
        magmaDoubleComplex *dB, magma_int_t lddb, 
        magmaDoubleComplex *U, magmaDoubleComplex *V, 
        magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_zgetrf_mic(
        magma_int_t m, magma_int_t n,
        magmaDoubleComplex_ptr dA, size_t dA_offset, magma_int_t ldda,
        magma_int_t *ipiv, magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_zgetrf_nopiv_mic(
        magma_int_t m, magma_int_t n,
        magmaDoubleComplex_ptr dA, size_t dA_offset, magma_int_t ldda,
        magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_zgetrf_mmic(
        magma_int_t num_mics, magma_int_t m, magma_int_t n,
        magmaDoubleComplex_ptr *dA, magma_int_t ldda,
        magma_int_t *ipiv, magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_zgeqrf2_mmic(
        magma_int_t num_mics, magma_int_t m, magma_int_t n,
        magmaDoubleComplex_ptr *dlA, magma_int_t ldda,
        magmaDoubleComplex *tau, magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_zhetrf_nopiv_mic(
        magma_uplo_t uplo, magma_int_t n,
        magmaDoubleComplex_ptr dA, size_t dA_offset, magma_int_t ldda,
        magma_queue_t queue,
        magma_int_t *info);

magma_int_t
magma_zlarfb_mic(
        magma_side_t side, magma_trans_t trans, magma_direct_t direct, magma_storev_t storev,
        magma_int_t m, magma_int_t n, magma_int_t k,
        magmaDoubleComplex_ptr dV, size_t dV_offset, magma_int_t ldv,
        magmaDoubleComplex_ptr dT, size_t dT_offset, magma_int_t ldt,
        magmaDoubleComplex_ptr dC, size_t dC_offset, magma_int_t ldc,
        magmaDoubleComplex_ptr dwork, size_t dwork_offset, magma_int_t ldwork,
        magma_queue_t queue);

magma_int_t
magma_zlarfb_mic_gemm(
        magma_side_t side, magma_trans_t trans, magma_direct_t direct, magma_storev_t storev,
        magma_int_t m, magma_int_t n, magma_int_t k,
        magmaDoubleComplex_ptr dV, size_t dV_offset, magma_int_t ldv,
        magmaDoubleComplex_ptr dT, size_t dT_offset, magma_int_t ldt,
        magmaDoubleComplex_ptr dC, size_t dC_offset, magma_int_t ldc,
        magmaDoubleComplex_ptr dwork, size_t dwork_offset, magma_int_t ldwork,
        magmaDoubleComplex_ptr dworkvt, size_t dworkvt_offset, magma_int_t ldworkvt,
        magma_queue_t queue);

magma_int_t
magma_zpotrf_mic(
        magma_uplo_t uplo,
        magma_int_t n,
        magmaDoubleComplex_ptr dA, size_t dA_offset, magma_int_t ldda,
        magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_zpotrf_mmic(
        magma_int_t num_mics,
        magma_uplo_t uplo,
        magma_int_t n,
        magmaDoubleComplex_ptr *dA, magma_int_t ldda,
        magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_zpotrs_mic(
         magma_uplo_t uplo, magma_int_t n, magma_int_t nrhs,
         magmaDoubleComplex_ptr dA, size_t dA_offset, magma_int_t ldda,
         magmaDoubleComplex_ptr dB, size_t dB_offset, magma_int_t lddb,
         magma_int_t *info, magma_queue_t queue );

magma_int_t
magma_zposv(
                magma_uplo_t uplo, magma_int_t n, magma_int_t nrhs,
                magmaDoubleComplex_ptr A, magma_int_t ldda,
                magmaDoubleComplex_ptr B, magma_int_t lddb,
                magma_int_t *info, magma_queue_t queue );

magma_int_t
magma_zposv_mic(
                magma_uplo_t uplo, magma_int_t n, magma_int_t nrhs,
                magmaDoubleComplex_ptr dA, size_t dA_offset, magma_int_t ldda,
                magmaDoubleComplex_ptr dB, size_t dB_offset, magma_int_t lddb,
                magma_int_t *info, magma_queue_t queue );

magma_int_t
magma_zgetri_mic( magma_int_t n,
                magmaDoubleComplex *dA, magma_int_t ldda, magma_int_t *ipiv,
                magmaDoubleComplex *dwork, magma_int_t lwork, magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_ztrtri_mic( magma_uplo_t uplo,  magma_diag_t diag, magma_int_t n,
                magmaDoubleComplex *dA, magma_int_t ldda, magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_zgetrs_mic(magma_trans_t trans, magma_int_t n, magma_int_t nrhs,
                 magmaDoubleComplex_ptr dA, size_t dA_offset, magma_int_t ldda,
                 magma_int_t *ipiv,
                 magmaDoubleComplex_ptr dB, size_t dB_offset, magma_int_t lddb,
                 magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_zgetrs_nopiv_mic(magma_trans_t trans, magma_int_t n, magma_int_t nrhs,
                 magmaDoubleComplex_ptr dA, size_t dA_offset, magma_int_t ldda,
                 magmaDoubleComplex_ptr dB, size_t dB_offset, magma_int_t lddb,
                 magma_int_t *info, magma_queue_t queue);
magma_int_t
magma_zgesv_rbt( magma_bool_t ref, magma_int_t n, magma_int_t nrhs,
                 magmaDoubleComplex *A, magma_int_t lda, 
                 magmaDoubleComplex *B, magma_int_t ldb, 
                 magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_zgesv(     magma_int_t n, magma_int_t nrhs,
                 magmaDoubleComplex *A, magma_int_t lda, magma_int_t *ipiv,
                 magmaDoubleComplex *B, magma_int_t ldb, magma_int_t *info, magma_queue_t queue);
magma_int_t
magma_zgesv_mic( magma_int_t n, magma_int_t nrhs,
                 magmaDoubleComplex_ptr dA, size_t dA_offset, magma_int_t ldda,
                 magma_int_t *ipiv,
                 magmaDoubleComplex_ptr dB, size_t dB_offset, magma_int_t lddb,
                 magma_int_t *info, magma_queue_t queue );

magma_int_t
magma_zgesv_nopiv_mic( magma_int_t n, magma_int_t nrhs,
                 magmaDoubleComplex_ptr dA, size_t dA_offset, magma_int_t ldda,
                 magmaDoubleComplex_ptr dB, size_t dB_offset, magma_int_t lddb,
                 magma_int_t *info, magma_queue_t queue );

magma_int_t
magma_zunmqr_mic(magma_side_t side, magma_trans_t trans,
                 magma_int_t m, magma_int_t n, magma_int_t k,
                 magmaDoubleComplex_ptr dA, size_t dA_offset, magma_int_t ldda,
                 magmaDoubleComplex *tau,
                 magmaDoubleComplex_ptr dC, size_t dC_offset, magma_int_t lddc,
                 magmaDoubleComplex *hwork, magma_int_t lwork,
                 magmaDoubleComplex_ptr dT, size_t dT_offset, magma_int_t nb,
                 magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_zgeqrs_mic(magma_int_t m, magma_int_t n, magma_int_t nrhs,
                 magmaDoubleComplex_ptr dA, size_t dA_offset, magma_int_t ldda,
                 magmaDoubleComplex *tau,   magmaDoubleComplex_ptr dT, size_t dT_offset,
                 magmaDoubleComplex_ptr dB, size_t dB_offset, magma_int_t lddb,
                 magmaDoubleComplex *hwork, magma_int_t lwork,
                 magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_zgeqrf_mic( magma_int_t m, magma_int_t n,
                  magmaDoubleComplex_ptr dA, size_t dA_offset,  magma_int_t ldda,
                  magmaDoubleComplex *tau, magmaDoubleComplex_ptr dT, size_t dT_offset,
                  magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_zgels_mic( magma_trans_t trans, magma_int_t m, magma_int_t n, magma_int_t nrhs,
                 magmaDoubleComplex_ptr dA, size_t dA_offset,  magma_int_t ldda,
                 magmaDoubleComplex_ptr dB, size_t dB_offset,  magma_int_t lddb,
                 magmaDoubleComplex *hwork, magma_int_t lwork,
                 magma_int_t *info, magma_queue_t queue );

magma_int_t
magma_zgehrd(   magma_int_t n, magma_int_t ilo, magma_int_t ihi,
                magmaDoubleComplex *a, magma_int_t lda,
                magmaDoubleComplex *tau,
                magmaDoubleComplex *work, magma_int_t lwork,
                magmaDoubleComplex_ptr dT, size_t dT_offset,
                magma_int_t *info, magma_queue_t queue );

magma_int_t
magma_zlabrd_mic( magma_int_t m, magma_int_t n, magma_int_t nb,
                  magmaDoubleComplex *a, magma_int_t lda,
                  magmaDoubleComplex_ptr da, size_t da_offset, magma_int_t ldda,
                  double *d, double *e, magmaDoubleComplex *tauq, magmaDoubleComplex *taup,
                  magmaDoubleComplex *x, magma_int_t ldx,
                  magmaDoubleComplex_ptr dx, size_t dx_offset, magma_int_t lddx,
                  magmaDoubleComplex *y, magma_int_t ldy,
                  magmaDoubleComplex_ptr dy, size_t dy_offset, magma_int_t lddy,
                  magma_int_t pstep, magma_queue_t queue );

magma_int_t
magma_zlahef_mic(
		 magma_uplo_t uplo, magma_int_t n, magma_int_t nb, magma_int_t *kb,
		 magmaDoubleComplex *hA, magma_int_t lda,
		 magmaDoubleComplex_ptr dA, size_t dA_offset, magma_int_t ldda,
		 magma_int_t *ipiv,
		 magmaDoubleComplex_ptr dW, size_t dW_offset, magma_int_t lddw,
		 magma_int_t *info,
		 magma_queue_t queue);

magma_int_t
zhetrf_nopiv_cpu(
                 magma_uplo_t uplo, magma_int_t n, magma_int_t ib,
                 magmaDoubleComplex *A, magma_int_t lda,
                 magma_int_t *info);

magma_int_t
magma_zlahr2(   magma_int_t n, magma_int_t k, magma_int_t nb,
                magmaDoubleComplex_ptr da, size_t da_offset, magmaDoubleComplex_ptr dv, size_t dv_offset,
                magmaDoubleComplex *a, magma_int_t lda,
                magmaDoubleComplex *tau, magmaDoubleComplex *t, magma_int_t ldt,
                magmaDoubleComplex *y, magma_int_t ldy,
                magma_queue_t queue     );

magma_int_t
magma_zlahru(   magma_int_t n, magma_int_t ihi, magma_int_t k, magma_int_t nb,
                magmaDoubleComplex *a, magma_int_t lda,
                magmaDoubleComplex_ptr d_a, size_t d_a_offset, magmaDoubleComplex_ptr y, size_t y_offset,
                magmaDoubleComplex_ptr v, size_t v_offset, magmaDoubleComplex_ptr d_t, size_t dt_offset,
                magmaDoubleComplex_ptr d_work, size_t d_work_offset, magma_queue_t queue );

magma_int_t
magma_zunghr(   magma_int_t n, magma_int_t ilo, magma_int_t ihi,
                magmaDoubleComplex *a, magma_int_t lda,
                magmaDoubleComplex *tau,
                magmaDoubleComplex_ptr dT, size_t dT_offset, magma_int_t nb,
                magma_int_t *info, magma_queue_t queue );

magma_int_t
magma_zungqr(   magma_int_t m, magma_int_t n, magma_int_t k,
                magmaDoubleComplex *a, magma_int_t lda,
                magmaDoubleComplex *tau, magmaDoubleComplex_ptr dT, size_t dT_offset,
                magma_int_t nb, magma_int_t *info, magma_queue_t queue );

magma_int_t
magma_zungqr2(  magma_int_t m, magma_int_t n, magma_int_t k,
                magmaDoubleComplex *A, magma_int_t lda,
                magmaDoubleComplex *tau,
                magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_zunmbr(
                magma_vect_t vect, magma_side_t side, magma_trans_t trans,
                magma_int_t m, magma_int_t n, magma_int_t k,
                magmaDoubleComplex *A, magma_int_t lda,
                magmaDoubleComplex *tau,
                magmaDoubleComplex *C, magma_int_t ldc,
                magmaDoubleComplex *work, magma_int_t lwork,
                magma_queue_t queue, magma_int_t *info);

magma_int_t
magma_zunmlq(
                magma_side_t side, magma_trans_t trans,
                magma_int_t m, magma_int_t n, magma_int_t k,
                magmaDoubleComplex *A, magma_int_t lda,
                magmaDoubleComplex *tau,
                magmaDoubleComplex *C, magma_int_t ldc,
                magmaDoubleComplex *work, magma_int_t lwork,
                magma_queue_t queue, magma_int_t *info);

magma_int_t
magma_zunmql(   magma_side_t side, magma_trans_t trans,
                magma_int_t m, magma_int_t n, magma_int_t k,
                magmaDoubleComplex *a, magma_int_t lda,
                magmaDoubleComplex *tau,
                magmaDoubleComplex *c, magma_int_t ldc,
                magmaDoubleComplex *work, magma_int_t lwork,
                magma_int_t *info, magma_queue_t queue );

magma_int_t
magma_zunmqr(   magma_side_t side, magma_trans_t trans,
                magma_int_t m, magma_int_t n, magma_int_t k,
                magmaDoubleComplex *a, magma_int_t lda, magmaDoubleComplex *tau,
                magmaDoubleComplex *c, magma_int_t ldc,
                magmaDoubleComplex *work, magma_int_t lwork,
                magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_zunmtr(   magma_side_t side, magma_uplo_t uplo, magma_trans_t trans,
                magma_int_t m, magma_int_t n,
                magmaDoubleComplex *a,    magma_int_t lda,
                magmaDoubleComplex *tau,
                magmaDoubleComplex *c,    magma_int_t ldc,
                magmaDoubleComplex *work, magma_int_t lwork,
                magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_zlatrd(magma_uplo_t uplo, magma_int_t n, magma_int_t nb,
             magmaDoubleComplex *a,  magma_int_t lda,
             double *e, magmaDoubleComplex *tau,
             magmaDoubleComplex *w,  magma_int_t ldw,
             magmaDoubleComplex *da, size_t da_offset, magma_int_t ldda,
             magmaDoubleComplex *dw, size_t dw_offset, magma_int_t lddw, 
             magma_int_t pstep, magma_queue_t queue);

magma_int_t
magma_zhetrd(  magma_uplo_t uplo, magma_int_t n,
               magmaDoubleComplex *a, magma_int_t lda,
               double *d, double *e, magmaDoubleComplex *tau,
               magmaDoubleComplex *work, magma_int_t lwork,
               magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_zhetrf(  magma_uplo_t uplo, magma_int_t n,
	       magmaDoubleComplex *A, magma_int_t lda,
	       magmaDoubleComplex_ptr da, magma_int_t ldda,
	       magma_int_t *ipiv, magma_int_t *d_ipiv,
	       magma_queue_t queue, magma_int_t *info );

magma_int_t
magma_zhetrf_nopiv(
               magma_uplo_t uplo, magma_int_t n,
               magmaDoubleComplex *A, magma_int_t lda,
               magma_queue_t queue,
               magma_int_t *info);


//##############################################################################
// MAGMA Sparse-Iter
//

magma_int_t
magma_zjacobidiagscale(
    magma_int_t n,
    magma_z_vector b,
    magma_z_vector d,
    magma_z_vector *c,
    magma_queue_t queue );

/* ////////////////////////////////////////////////////////////////////////////
 -- MAGMA utility function definitions
*/

void magma_zprint    ( magma_int_t m, magma_int_t n, magmaDoubleComplex     *A, magma_int_t lda  );
void magma_zprint_gpu( magma_int_t m, magma_int_t n, magmaDoubleComplex_ptr dA, magma_int_t ldda );

void zpanel_to_q(magma_uplo_t uplo, magma_int_t ib, magmaDoubleComplex *A, magma_int_t lda, magmaDoubleComplex *work);
void zq_to_panel(magma_uplo_t uplo, magma_int_t ib, magmaDoubleComplex *A, magma_int_t lda, magmaDoubleComplex *work);

#ifdef __cplusplus
}
#endif

#undef PRECISION_z
#endif /* MAGMA_Z_H */
