/*
 *   -- clMAGMA (version 1.3.1) --
 *      Univ. of Tennessee, Knoxville
 *      Univ. of California, Berkeley
 *      Univ. of Colorado, Denver
 *      @date January 2015
 *
 * @author Mark Gates
 * @generated from magmablas_z.h normal z -> c, Fri Jan 30 18:21:50 2015
 */

#ifndef MAGMA_BLAS_C_H
#define MAGMA_BLAS_C_H

#include "magma_types.h"

#ifdef __cplusplus
extern "C" {
#endif


magma_int_t
magma_csetvector(
    magma_int_t n,
    magmaFloatComplex const* hx_src, magma_int_t incx,
    magmaFloatComplex*       dy_dst, magma_int_t incy,
    magma_queue_t queue );

magma_int_t
magma_cgetvector(
   magma_int_t n,
   magmaFloatComplex const* dx_src, magma_int_t incx,
   magmaFloatComplex*       hy_dst, magma_int_t incy,
   magma_queue_t queue );

magma_int_t
magma_csetvector_async(
    magma_int_t n,
    magmaFloatComplex const* hx_src, magma_int_t incx,
    magmaFloatComplex*       dy_dst, magma_int_t incy,
    magma_queue_t queue, magma_event_t *event );

magma_int_t
magma_cgetvector_async(
    magma_int_t m, magma_int_t n,
    magmaFloatComplex_const_ptr dA_src, size_t dA_offset, magma_int_t ldda,
    magmaFloatComplex*          hA_dst, size_t hA_offset, magma_int_t ldha,
    magma_queue_t queue, magma_event_t *event );


// ========================================
// copying sub-matrices (contiguous columns)
magma_int_t
magma_csetmatrix(
    magma_int_t m, magma_int_t n,
    magmaFloatComplex const* hA_src, size_t hA_offset, magma_int_t ldha,
    magmaFloatComplex_ptr    dA_dst, size_t dA_offset, magma_int_t ldda,
    magma_queue_t queue );

magma_int_t
magma_cgetmatrix(
    magma_int_t m, magma_int_t n,
    magmaFloatComplex_const_ptr dA_src, size_t dA_offset, magma_int_t ldda,
    magmaFloatComplex*          hA_dst, size_t hA_offset, magma_int_t ldha,
    magma_queue_t queue );

magma_int_t
magma_csetmatrix_async(
    magma_int_t m, magma_int_t n,
    magmaFloatComplex const* hA_src, size_t hA_offset, magma_int_t ldha,
    magmaFloatComplex_ptr    dA_dst, size_t dA_offset, magma_int_t ldda,
    magma_queue_t queue, magma_event_t *event );

magma_int_t
magma_cgetmatrix_async(
    magma_int_t m, magma_int_t n,
    magmaFloatComplex_const_ptr dA_src, size_t dA_offset, magma_int_t ldda,
    magmaFloatComplex*          hA_dst, size_t hA_offset, magma_int_t ldha,
    magma_queue_t queue, magma_event_t *event );

magma_int_t
magma_csetintarray_async(
    magma_int_t lda, magma_int_t const* hA_src, size_t hA_offset, magma_int_t *dA_dst, size_t dA_offset, magma_queue_t queue, magma_event_t *event );

magma_int_t
magma_cgetintarray_async(
    magma_int_t lda, magma_int_t const* hA_src, size_t hA_offset, magma_int_t *dA_dst, size_t dA_offset, magma_queue_t queue, magma_event_t *event );


// ========================================
// matrix transpose and swapping functions
//magma_int_t
//magma_cinplace_transpose(
//    magmaFloatComplex_ptr dA, size_t dA_offset, magma_int_t lda, magma_int_t n,
//    magma_queue_t queue );

#define magma_cinplace_transpose( dAT, dAT_offset, ldda, lddat, queue ) ;

//magma_int_t
//magma_ctranspose2(
//    magmaFloatComplex_ptr odata, size_t odata_offset, magma_int_t ldo,
//    magmaFloatComplex_ptr idata, size_t idata_offset, magma_int_t ldi,
//    magma_int_t m, magma_int_t n,
//    magma_queue_t queue );

#define magma_ctranspose2( dAT, dAT_offset, lddat, dA, dA_offset,  ldda, m, n, queue ) \
    magma_ctranspose(dAT, 0, lddat, dA, 0, ldda, m, n, queue)

magma_int_t
magma_ctranspose(
    magmaFloatComplex_ptr odata, magma_int_t offo, magma_int_t ldo,
    magmaFloatComplex_ptr idata, magma_int_t offi, magma_int_t ldi,
    magma_int_t m, magma_int_t n,
    magma_queue_t queue );

magma_int_t
magma_cswap(magma_int_t n, magmaFloatComplex *dA1T, magma_int_t lda1, magmaFloatComplex *dA2T, magma_int_t lda2, magma_queue_t queue);

magma_int_t
magma_claswp(magma_int_t n, magmaFloatComplex_ptr d_A, magma_int_t ldda, magma_int_t k1, magma_int_t k2, magma_int_t *d_ipiv, magma_int_t incx, magma_queue_t queue);

// ========================================
// BLAS functions

magma_int_t
magma_cscal(
    magma_int_t n,
    magmaFloatComplex alpha,
    magmaFloatComplex *dx, 
    magma_int_t incx,
    magma_queue_t handle );

magma_int_t
magma_ccsrgemv(
    magma_trans_t  trans,
    magma_int_t n,
    magmaFloatComplex_const_ptr daval,
    magmaIndex_const_ptr darow,
    magmaIndex_const_ptr dacol,
    magmaFloatComplex_const_ptr dx,
    magmaFloatComplex_ptr dy,
    magma_queue_t handle );

magma_int_t
magma_ccsrgespmm(
    magma_trans_t  trans,
    magma_int_t request,
    magma_int_t sort,
    magma_int_t m,
    magma_int_t n,
    magma_int_t k,
    magmaFloatComplex alpha,
    magmaFloatComplex_const_ptr daval,
    magmaIndex_const_ptr darow,
    magmaIndex_const_ptr dacol,
    magmaFloatComplex_const_ptr dbval,
    magmaIndex_const_ptr dbrow,
    magmaIndex_const_ptr dbcol,
    magmaFloatComplex_ptr dcval,
    magmaIndex_ptr dcrow,
    magmaIndex_ptr dccol,
    magma_int_t nzmax,
    magma_int_t info,
    magma_queue_t handle );

magma_int_t
magma_caxpy(
    magma_int_t n,
    magmaFloatComplex alpha,
    magmaFloatComplex_const_ptr dx,
    magma_int_t incx,
    magmaFloatComplex_ptr dy,
    magma_int_t incy,
    magma_queue_t handle );

magma_int_t
magma_ccopy(
    magma_int_t n,
    magmaFloatComplex_const_ptr dx, magma_int_t incx,
    magmaFloatComplex_ptr dy, magma_int_t incy,
    magma_queue_t queue );

magmaFloatComplex
magma_cdotc(
    magma_int_t n,
    magmaFloatComplex_const_ptr dx, magma_int_t incx,
    magmaFloatComplex_const_ptr dy, magma_int_t incy,
    magma_queue_t queue );

float
magma_scnrm2(
    magma_int_t n,
    magmaFloatComplex_const_ptr dx, magma_int_t incx,
    magma_queue_t queue );

magma_int_t
magma_ccopyvector(
    magma_int_t n,
    magmaFloatComplex_const_ptr dx_src, magma_int_t incx,
    magmaFloatComplex_ptr dy_dst, magma_int_t incy,
    magma_queue_t handle );

magma_int_t
magma_cgemm(
    magma_trans_t transA, magma_trans_t transB,
    magma_int_t m, magma_int_t n, magma_int_t k,
    magmaFloatComplex alpha, magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloatComplex_const_ptr dB, size_t dB_offset, magma_int_t ldb,
    magmaFloatComplex beta,  magmaFloatComplex_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_queue_t queue );

magma_int_t
magma_cgemv(
    magma_trans_t transA,
    magma_int_t m, magma_int_t n,
    magmaFloatComplex alpha, magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloatComplex_const_ptr dx, size_t dx_offset, magma_int_t incx,
    magmaFloatComplex beta,  magmaFloatComplex_ptr       dy, size_t dy_offset, magma_int_t incy,
    magma_queue_t queue );

magma_int_t
magma_cprbt(
    magma_int_t n, magmaFloatComplex_ptr dA, magma_int_t lda, 
    magmaFloatComplex_const_ptr du, magmaFloatComplex_const_ptr dv,
    magma_queue_t queue );

magma_int_t
magma_cprbt_mv(
    magma_int_t n, magmaFloatComplex_const_ptr dv, magmaFloatComplex_ptr db, 
    magma_queue_t queue );

magma_int_t
magma_cprbt_mtv(
    magma_int_t n, magmaFloatComplex_const_ptr du, magmaFloatComplex_ptr db, 
    magma_queue_t queue );

magma_int_t
magma_chemm(
    magma_side_t side, magma_uplo_t uplo,
    magma_int_t m, magma_int_t n,
    magmaFloatComplex alpha, magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloatComplex_const_ptr dB, size_t dB_offset, magma_int_t ldb,
    magmaFloatComplex beta,  magmaFloatComplex_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_queue_t queue );

magma_int_t
magma_chemm_ours(
    magma_side_t side, magma_uplo_t uplo,
    magma_int_t m, magma_int_t n,
    magmaFloatComplex alpha, magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloatComplex_const_ptr dB, size_t dB_offset, magma_int_t ldb,
    magmaFloatComplex beta,  magmaFloatComplex_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_int_t globaldAoffset,
    magma_queue_t queue );

magma_int_t
magma_chemv(
    magma_uplo_t uplo,
    magma_int_t n,
    magmaFloatComplex alpha, magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloatComplex_const_ptr dx, size_t dx_offset, magma_int_t incx,
    magmaFloatComplex beta,  magmaFloatComplex_ptr       dy, size_t dy_offset, magma_int_t incy,
    magma_queue_t queue );

magma_int_t
magma_csymv2(
    magma_uplo_t uplo,
    magma_int_t n,
    magmaFloatComplex alpha, magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloatComplex_const_ptr dx, size_t dx_offset, magma_int_t incx,
    magmaFloatComplex beta,  magmaFloatComplex_ptr       dy, size_t dy_offset, magma_int_t incy,
    magma_queue_t queue );


magma_int_t
magma_cherk(
    magma_uplo_t uplo, magma_trans_t trans,
    magma_int_t n, magma_int_t k,
    float alpha, magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
    float beta,  magmaFloatComplex_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_queue_t queue );

void magmablas_cherk_mmic2(
    magma_uplo_t uplo, magma_trans_t trans, magma_int_t n, magma_int_t k,
    float alpha, magmaFloatComplex *dA[], magma_int_t lda, magma_int_t aoff,
    float beta,  magmaFloatComplex *dC[], magma_int_t ldc, magma_int_t offset,
    magma_int_t nmic, magma_int_t nb, magma_queue_t queue);
 
magma_int_t
magma_cherk_mmic(
    magma_uplo_t uplo, magma_trans_t trans, magma_int_t n, magma_int_t k,
    float alpha, magmaFloatComplex_const_ptr dA, magma_int_t lda, magma_int_t dAoff,
    float beta,  magmaFloatComplex_const_ptr dC, magma_int_t ldc, magma_int_t offset,
    magma_int_t nmic, magma_int_t nb, magma_int_t my_id);

magma_int_t
magma_cher2k(
    magma_uplo_t uplo, magma_trans_t trans,
    magma_int_t n, magma_int_t k,
    magmaFloatComplex alpha, magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
    magmaFloatComplex_const_ptr dB, size_t dB_offset, magma_int_t ldb,
    float beta,  magmaFloatComplex_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_queue_t queue );

void magmablas_cher2k_mmic2(
    magma_uplo_t uplo, magma_trans_t trans, magma_int_t n, magma_int_t k,
    magmaFloatComplex alpha, magmaFloatComplex *dA[], magma_int_t lda, magma_int_t aoff,
    magmaFloatComplex *dB[], magma_int_t ldb, magma_int_t boff,
    float beta,           magmaFloatComplex *dC[], magma_int_t ldc,  magma_int_t offset,
    magma_int_t nmic, magma_int_t nb, magma_queue_t queue);

magma_int_t
magma_cher2k_mmic(
    magma_uplo_t uplo, magma_trans_t trans, magma_int_t n, magma_int_t k,
    magmaFloatComplex alpha, magmaFloatComplex_const_ptr dA, magma_int_t lda, magma_int_t dAoff,
    magmaFloatComplex_const_ptr dB, magma_int_t ldb, magma_int_t dBoff,
    float beta,           magmaFloatComplex_const_ptr dC, magma_int_t ldc,  magma_int_t offset,
    magma_int_t nmic, magma_int_t nb, magma_int_t my_id);

magma_int_t
magma_ctrsm(
    magma_side_t side, magma_uplo_t uplo, magma_trans_t trans, magma_diag_t diag,
    magma_int_t m, magma_int_t n,
    magmaFloatComplex alpha, magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloatComplex_ptr       dB, size_t dB_offset, magma_int_t ldb,
    magma_queue_t queue );

magma_int_t
magma_ctrsv(
    magma_uplo_t uplo, magma_trans_t trans, magma_diag_t diag,
    magma_int_t n,
    magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
    magmaFloatComplex_ptr dx, size_t dx_offset, magma_int_t incx,
    magma_queue_t queue );

magma_int_t
magma_ctrmm(
    magma_side_t side, magma_uplo_t uplo, magma_trans_t trans, magma_diag_t diag,
    magma_int_t m, magma_int_t n,
    magmaFloatComplex alpha, magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloatComplex_ptr       dB, size_t dB_offset, magma_int_t ldb,
    magma_queue_t queue );

magma_int_t
magma_ctrmv(
    magma_uplo_t uplo, magma_trans_t trans, magma_diag_t diag,
    magma_int_t n,
    magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
    magmaFloatComplex_ptr dx, size_t dx_offset, magma_int_t incx,
    magma_queue_t queue );



magma_int_t
magma_clarfb( magma_side_t side, magma_trans_t trans, magma_direct_t direct, magma_storev_t storev,
              magma_int_t m, magma_int_t n, magma_int_t k,
              magmaFloatComplex_ptr dV, size_t dV_offset, magma_int_t ldv,
              magmaFloatComplex_ptr dT, size_t dT_offset, magma_int_t ldt,
              magmaFloatComplex_ptr dC, size_t dC_offset, magma_int_t ldc,
              magmaFloatComplex_ptr dwork, size_t dwork_offset, magma_int_t ldwork,
              magma_queue_t queue);

magma_int_t
magma_clarfb_gemm(magma_side_t side, magma_trans_t trans, magma_direct_t direct, magma_storev_t storev,
              magma_int_t m, magma_int_t n, magma_int_t k,
              magmaFloatComplex_ptr dV, size_t dV_offset, magma_int_t ldv,
              magmaFloatComplex_ptr dT, size_t dT_offset, magma_int_t ldt,
              magmaFloatComplex_ptr dC, size_t dC_offset, magma_int_t ldc,
              magmaFloatComplex_ptr dwork, size_t dwork_offset, magma_int_t ldwork,
              magmaFloatComplex_ptr dworkvt, size_t dworkvt_offset, magma_int_t ldworkvt,
              magma_queue_t queue);

magma_int_t
magmablas_clascl_diag(
              magma_type_t uplo, magma_int_t m, magma_int_t n,
              magmaFloatComplex_ptr dD, magma_int_t lddd,
              magmaFloatComplex_ptr dA, magma_int_t ldda,
              magma_queue_t queue );

magma_int_t
magma_claset(magma_uplo_t uplo,
             magma_int_t m, magma_int_t n,
             magmaFloatComplex alpha, magmaFloatComplex beta,
             magmaFloatComplex_ptr dA, size_t dA_offset, magma_int_t lda,
             magma_queue_t queue);

// ========================================
// BLAS functions -- combined for MIC
magma_int_t
magma_ctrsmgemm(
    magma_side_t side, magma_uplo_t uplo, magma_trans_t trans, magma_diag_t diag,
    magma_int_t tm, magma_int_t tn,
    magmaFloatComplex talpha, magmaFloatComplex_const_ptr tdA, size_t tdA_offset, magma_int_t tlda,
                              magmaFloatComplex_ptr       tdB, size_t tdB_offset, magma_int_t tldb,
    magma_trans_t transA, magma_trans_t transB,
    magma_int_t m, magma_int_t n, magma_int_t k,
    magmaFloatComplex alpha, magmaFloatComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloatComplex_const_ptr dB, size_t dB_offset, magma_int_t ldb,
    magmaFloatComplex beta,  magmaFloatComplex_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_queue_t queue );


float
magmablas_clange(
    magma_norm_t norm, magma_int_t m, magma_int_t n,
    magmaFloatComplex_const_ptr dA, magma_int_t ldda,
    magmaFloat_ptr dwork,
    magma_queue_t queue);

magma_int_t
magmablas_clacpy(
    magma_uplo_t uplo, magma_int_t m, magma_int_t n,
    magmaFloatComplex_const_ptr dA, magma_int_t ldda,
    magmaFloatComplex_ptr dB, magma_int_t lddb,
    magma_queue_t queue);

magma_int_t
magma_icamax(
    magma_int_t n,
    magmaFloatComplex_const_ptr dx, magma_int_t incx,
    magma_queue_t queue);

magma_int_t
magmablas_caxpycp2(
    magma_int_t m,
    magmaFloatComplex_ptr r, magmaFloatComplex_ptr x, magmaFloatComplex_ptr b,
    magma_queue_t queue);






#ifdef __cplusplus
}
#endif

#endif        //  #ifndef MAGMA_BLAS_H
