/*
 *   -- clMAGMA (version 1.3.1) --
 *      Univ. of Tennessee, Knoxville
 *      Univ. of California, Berkeley
 *      Univ. of Colorado, Denver
 *      @date January 2015
 *
 * @author Mark Gates
 * @precisions normal z -> s d c
 */

#ifndef MAGMA_BLAS_Z_H
#define MAGMA_BLAS_Z_H

#include "magma_types.h"

#ifdef __cplusplus
extern "C" {
#endif


magma_int_t
magma_zsetvector(
    magma_int_t n,
    magmaDoubleComplex const* hx_src, magma_int_t incx,
    magmaDoubleComplex*       dy_dst, magma_int_t incy,
    magma_queue_t queue );

magma_int_t
magma_zgetvector(
   magma_int_t n,
   magmaDoubleComplex const* dx_src, magma_int_t incx,
   magmaDoubleComplex*       hy_dst, magma_int_t incy,
   magma_queue_t queue );

magma_int_t
magma_zsetvector_async(
    magma_int_t n,
    magmaDoubleComplex const* hx_src, magma_int_t incx,
    magmaDoubleComplex*       dy_dst, magma_int_t incy,
    magma_queue_t queue, magma_event_t *event );

magma_int_t
magma_zgetvector_async(
    magma_int_t m, magma_int_t n,
    magmaDoubleComplex_const_ptr dA_src, size_t dA_offset, magma_int_t ldda,
    magmaDoubleComplex*          hA_dst, size_t hA_offset, magma_int_t ldha,
    magma_queue_t queue, magma_event_t *event );


// ========================================
// copying sub-matrices (contiguous columns)
magma_int_t
magma_zsetmatrix(
    magma_int_t m, magma_int_t n,
    magmaDoubleComplex const* hA_src, size_t hA_offset, magma_int_t ldha,
    magmaDoubleComplex_ptr    dA_dst, size_t dA_offset, magma_int_t ldda,
    magma_queue_t queue );

magma_int_t
magma_zgetmatrix(
    magma_int_t m, magma_int_t n,
    magmaDoubleComplex_const_ptr dA_src, size_t dA_offset, magma_int_t ldda,
    magmaDoubleComplex*          hA_dst, size_t hA_offset, magma_int_t ldha,
    magma_queue_t queue );

magma_int_t
magma_zsetmatrix_async(
    magma_int_t m, magma_int_t n,
    magmaDoubleComplex const* hA_src, size_t hA_offset, magma_int_t ldha,
    magmaDoubleComplex_ptr    dA_dst, size_t dA_offset, magma_int_t ldda,
    magma_queue_t queue, magma_event_t *event );

magma_int_t
magma_zgetmatrix_async(
    magma_int_t m, magma_int_t n,
    magmaDoubleComplex_const_ptr dA_src, size_t dA_offset, magma_int_t ldda,
    magmaDoubleComplex*          hA_dst, size_t hA_offset, magma_int_t ldha,
    magma_queue_t queue, magma_event_t *event );

magma_int_t
magma_zsetintarray_async(
    magma_int_t lda, magma_int_t const* hA_src, size_t hA_offset, magma_int_t *dA_dst, size_t dA_offset, magma_queue_t queue, magma_event_t *event );

magma_int_t
magma_zgetintarray_async(
    magma_int_t lda, magma_int_t const* hA_src, size_t hA_offset, magma_int_t *dA_dst, size_t dA_offset, magma_queue_t queue, magma_event_t *event );


// ========================================
// matrix transpose and swapping functions
//magma_int_t
//magma_zinplace_transpose(
//    magmaDoubleComplex_ptr dA, size_t dA_offset, magma_int_t lda, magma_int_t n,
//    magma_queue_t queue );

#define magma_zinplace_transpose( dAT, dAT_offset, ldda, lddat, queue ) ;

//magma_int_t
//magma_ztranspose2(
//    magmaDoubleComplex_ptr odata, size_t odata_offset, magma_int_t ldo,
//    magmaDoubleComplex_ptr idata, size_t idata_offset, magma_int_t ldi,
//    magma_int_t m, magma_int_t n,
//    magma_queue_t queue );

#define magma_ztranspose2( dAT, dAT_offset, lddat, dA, dA_offset,  ldda, m, n, queue ) \
    magma_ztranspose(dAT, 0, lddat, dA, 0, ldda, m, n, queue)

magma_int_t
magma_ztranspose(
    magmaDoubleComplex_ptr odata, magma_int_t offo, magma_int_t ldo,
    magmaDoubleComplex_ptr idata, magma_int_t offi, magma_int_t ldi,
    magma_int_t m, magma_int_t n,
    magma_queue_t queue );

magma_int_t
magma_zswap(magma_int_t n, magmaDoubleComplex *dA1T, magma_int_t lda1, magmaDoubleComplex *dA2T, magma_int_t lda2, magma_queue_t queue);

magma_int_t
magma_zlaswp(magma_int_t n, magmaDoubleComplex_ptr d_A, magma_int_t ldda, magma_int_t k1, magma_int_t k2, magma_int_t *d_ipiv, magma_int_t incx, magma_queue_t queue);

// ========================================
// BLAS functions

magma_int_t
magma_zscal(
    magma_int_t n,
    magmaDoubleComplex alpha,
    magmaDoubleComplex *dx, 
    magma_int_t incx,
    magma_queue_t handle );

magma_int_t
magma_zcsrgemv(
    magma_trans_t  trans,
    magma_int_t n,
    magmaDoubleComplex_const_ptr daval,
    magmaIndex_const_ptr darow,
    magmaIndex_const_ptr dacol,
    magmaDoubleComplex_const_ptr dx,
    magmaDoubleComplex_ptr dy,
    magma_queue_t handle );

magma_int_t
magma_zcsrgespmm(
    magma_trans_t  trans,
    magma_int_t request,
    magma_int_t sort,
    magma_int_t m,
    magma_int_t n,
    magma_int_t k,
    magmaDoubleComplex alpha,
    magmaDoubleComplex_const_ptr daval,
    magmaIndex_const_ptr darow,
    magmaIndex_const_ptr dacol,
    magmaDoubleComplex_const_ptr dbval,
    magmaIndex_const_ptr dbrow,
    magmaIndex_const_ptr dbcol,
    magmaDoubleComplex_ptr dcval,
    magmaIndex_ptr dcrow,
    magmaIndex_ptr dccol,
    magma_int_t nzmax,
    magma_int_t info,
    magma_queue_t handle );

magma_int_t
magma_zaxpy(
    magma_int_t n,
    magmaDoubleComplex alpha,
    magmaDoubleComplex_const_ptr dx,
    magma_int_t incx,
    magmaDoubleComplex_ptr dy,
    magma_int_t incy,
    magma_queue_t handle );

magma_int_t
magma_zcopy(
    magma_int_t n,
    magmaDoubleComplex_const_ptr dx, magma_int_t incx,
    magmaDoubleComplex_ptr dy, magma_int_t incy,
    magma_queue_t queue );

magmaDoubleComplex
magma_zdotc(
    magma_int_t n,
    magmaDoubleComplex_const_ptr dx, magma_int_t incx,
    magmaDoubleComplex_const_ptr dy, magma_int_t incy,
    magma_queue_t queue );

double
magma_dznrm2(
    magma_int_t n,
    magmaDoubleComplex_const_ptr dx, magma_int_t incx,
    magma_queue_t queue );

magma_int_t
magma_zcopyvector(
    magma_int_t n,
    magmaDoubleComplex_const_ptr dx_src, magma_int_t incx,
    magmaDoubleComplex_ptr dy_dst, magma_int_t incy,
    magma_queue_t handle );

magma_int_t
magma_zgemm(
    magma_trans_t transA, magma_trans_t transB,
    magma_int_t m, magma_int_t n, magma_int_t k,
    magmaDoubleComplex alpha, magmaDoubleComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaDoubleComplex_const_ptr dB, size_t dB_offset, magma_int_t ldb,
    magmaDoubleComplex beta,  magmaDoubleComplex_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_queue_t queue );

magma_int_t
magma_zgemv(
    magma_trans_t transA,
    magma_int_t m, magma_int_t n,
    magmaDoubleComplex alpha, magmaDoubleComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaDoubleComplex_const_ptr dx, size_t dx_offset, magma_int_t incx,
    magmaDoubleComplex beta,  magmaDoubleComplex_ptr       dy, size_t dy_offset, magma_int_t incy,
    magma_queue_t queue );

magma_int_t
magma_zprbt(
    magma_int_t n, magmaDoubleComplex_ptr dA, magma_int_t lda, 
    magmaDoubleComplex_const_ptr du, magmaDoubleComplex_const_ptr dv,
    magma_queue_t queue );

magma_int_t
magma_zprbt_mv(
    magma_int_t n, magmaDoubleComplex_const_ptr dv, magmaDoubleComplex_ptr db, 
    magma_queue_t queue );

magma_int_t
magma_zprbt_mtv(
    magma_int_t n, magmaDoubleComplex_const_ptr du, magmaDoubleComplex_ptr db, 
    magma_queue_t queue );

magma_int_t
magma_zhemm(
    magma_side_t side, magma_uplo_t uplo,
    magma_int_t m, magma_int_t n,
    magmaDoubleComplex alpha, magmaDoubleComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaDoubleComplex_const_ptr dB, size_t dB_offset, magma_int_t ldb,
    magmaDoubleComplex beta,  magmaDoubleComplex_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_queue_t queue );

magma_int_t
magma_zhemm_ours(
    magma_side_t side, magma_uplo_t uplo,
    magma_int_t m, magma_int_t n,
    magmaDoubleComplex alpha, magmaDoubleComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaDoubleComplex_const_ptr dB, size_t dB_offset, magma_int_t ldb,
    magmaDoubleComplex beta,  magmaDoubleComplex_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_int_t globaldAoffset,
    magma_queue_t queue );

magma_int_t
magma_zhemv(
    magma_uplo_t uplo,
    magma_int_t n,
    magmaDoubleComplex alpha, magmaDoubleComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaDoubleComplex_const_ptr dx, size_t dx_offset, magma_int_t incx,
    magmaDoubleComplex beta,  magmaDoubleComplex_ptr       dy, size_t dy_offset, magma_int_t incy,
    magma_queue_t queue );

magma_int_t
magma_zsymv2(
    magma_uplo_t uplo,
    magma_int_t n,
    magmaDoubleComplex alpha, magmaDoubleComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaDoubleComplex_const_ptr dx, size_t dx_offset, magma_int_t incx,
    magmaDoubleComplex beta,  magmaDoubleComplex_ptr       dy, size_t dy_offset, magma_int_t incy,
    magma_queue_t queue );


magma_int_t
magma_zherk(
    magma_uplo_t uplo, magma_trans_t trans,
    magma_int_t n, magma_int_t k,
    double alpha, magmaDoubleComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
    double beta,  magmaDoubleComplex_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_queue_t queue );

void magmablas_zherk_mmic2(
    magma_uplo_t uplo, magma_trans_t trans, magma_int_t n, magma_int_t k,
    double alpha, magmaDoubleComplex *dA[], magma_int_t lda, magma_int_t aoff,
    double beta,  magmaDoubleComplex *dC[], magma_int_t ldc, magma_int_t offset,
    magma_int_t nmic, magma_int_t nb, magma_queue_t queue);
 
magma_int_t
magma_zherk_mmic(
    magma_uplo_t uplo, magma_trans_t trans, magma_int_t n, magma_int_t k,
    double alpha, magmaDoubleComplex_const_ptr dA, magma_int_t lda, magma_int_t dAoff,
    double beta,  magmaDoubleComplex_const_ptr dC, magma_int_t ldc, magma_int_t offset,
    magma_int_t nmic, magma_int_t nb, magma_int_t my_id);

magma_int_t
magma_zher2k(
    magma_uplo_t uplo, magma_trans_t trans,
    magma_int_t n, magma_int_t k,
    magmaDoubleComplex alpha, magmaDoubleComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
    magmaDoubleComplex_const_ptr dB, size_t dB_offset, magma_int_t ldb,
    double beta,  magmaDoubleComplex_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_queue_t queue );

void magmablas_zher2k_mmic2(
    magma_uplo_t uplo, magma_trans_t trans, magma_int_t n, magma_int_t k,
    magmaDoubleComplex alpha, magmaDoubleComplex *dA[], magma_int_t lda, magma_int_t aoff,
    magmaDoubleComplex *dB[], magma_int_t ldb, magma_int_t boff,
    double beta,           magmaDoubleComplex *dC[], magma_int_t ldc,  magma_int_t offset,
    magma_int_t nmic, magma_int_t nb, magma_queue_t queue);

magma_int_t
magma_zher2k_mmic(
    magma_uplo_t uplo, magma_trans_t trans, magma_int_t n, magma_int_t k,
    magmaDoubleComplex alpha, magmaDoubleComplex_const_ptr dA, magma_int_t lda, magma_int_t dAoff,
    magmaDoubleComplex_const_ptr dB, magma_int_t ldb, magma_int_t dBoff,
    double beta,           magmaDoubleComplex_const_ptr dC, magma_int_t ldc,  magma_int_t offset,
    magma_int_t nmic, magma_int_t nb, magma_int_t my_id);

magma_int_t
magma_ztrsm(
    magma_side_t side, magma_uplo_t uplo, magma_trans_t trans, magma_diag_t diag,
    magma_int_t m, magma_int_t n,
    magmaDoubleComplex alpha, magmaDoubleComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaDoubleComplex_ptr       dB, size_t dB_offset, magma_int_t ldb,
    magma_queue_t queue );

magma_int_t
magma_ztrsv(
    magma_uplo_t uplo, magma_trans_t trans, magma_diag_t diag,
    magma_int_t n,
    magmaDoubleComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
    magmaDoubleComplex_ptr dx, size_t dx_offset, magma_int_t incx,
    magma_queue_t queue );

magma_int_t
magma_ztrmm(
    magma_side_t side, magma_uplo_t uplo, magma_trans_t trans, magma_diag_t diag,
    magma_int_t m, magma_int_t n,
    magmaDoubleComplex alpha, magmaDoubleComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaDoubleComplex_ptr       dB, size_t dB_offset, magma_int_t ldb,
    magma_queue_t queue );

magma_int_t
magma_ztrmv(
    magma_uplo_t uplo, magma_trans_t trans, magma_diag_t diag,
    magma_int_t n,
    magmaDoubleComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
    magmaDoubleComplex_ptr dx, size_t dx_offset, magma_int_t incx,
    magma_queue_t queue );



magma_int_t
magma_zlarfb( magma_side_t side, magma_trans_t trans, magma_direct_t direct, magma_storev_t storev,
              magma_int_t m, magma_int_t n, magma_int_t k,
              magmaDoubleComplex_ptr dV, size_t dV_offset, magma_int_t ldv,
              magmaDoubleComplex_ptr dT, size_t dT_offset, magma_int_t ldt,
              magmaDoubleComplex_ptr dC, size_t dC_offset, magma_int_t ldc,
              magmaDoubleComplex_ptr dwork, size_t dwork_offset, magma_int_t ldwork,
              magma_queue_t queue);

magma_int_t
magma_zlarfb_gemm(magma_side_t side, magma_trans_t trans, magma_direct_t direct, magma_storev_t storev,
              magma_int_t m, magma_int_t n, magma_int_t k,
              magmaDoubleComplex_ptr dV, size_t dV_offset, magma_int_t ldv,
              magmaDoubleComplex_ptr dT, size_t dT_offset, magma_int_t ldt,
              magmaDoubleComplex_ptr dC, size_t dC_offset, magma_int_t ldc,
              magmaDoubleComplex_ptr dwork, size_t dwork_offset, magma_int_t ldwork,
              magmaDoubleComplex_ptr dworkvt, size_t dworkvt_offset, magma_int_t ldworkvt,
              magma_queue_t queue);

magma_int_t
magmablas_zlascl_diag(
              magma_type_t uplo, magma_int_t m, magma_int_t n,
              magmaDoubleComplex_ptr dD, magma_int_t lddd,
              magmaDoubleComplex_ptr dA, magma_int_t ldda,
              magma_queue_t queue );

magma_int_t
magma_zlaset(magma_uplo_t uplo,
             magma_int_t m, magma_int_t n,
             magmaDoubleComplex alpha, magmaDoubleComplex beta,
             magmaDoubleComplex_ptr dA, size_t dA_offset, magma_int_t lda,
             magma_queue_t queue);

// ========================================
// BLAS functions -- combined for MIC
magma_int_t
magma_ztrsmgemm(
    magma_side_t side, magma_uplo_t uplo, magma_trans_t trans, magma_diag_t diag,
    magma_int_t tm, magma_int_t tn,
    magmaDoubleComplex talpha, magmaDoubleComplex_const_ptr tdA, size_t tdA_offset, magma_int_t tlda,
                              magmaDoubleComplex_ptr       tdB, size_t tdB_offset, magma_int_t tldb,
    magma_trans_t transA, magma_trans_t transB,
    magma_int_t m, magma_int_t n, magma_int_t k,
    magmaDoubleComplex alpha, magmaDoubleComplex_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaDoubleComplex_const_ptr dB, size_t dB_offset, magma_int_t ldb,
    magmaDoubleComplex beta,  magmaDoubleComplex_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_queue_t queue );


double
magmablas_zlange(
    magma_norm_t norm, magma_int_t m, magma_int_t n,
    magmaDoubleComplex_const_ptr dA, magma_int_t ldda,
    magmaDouble_ptr dwork,
    magma_queue_t queue);

magma_int_t
magmablas_zlacpy(
    magma_uplo_t uplo, magma_int_t m, magma_int_t n,
    magmaDoubleComplex_const_ptr dA, magma_int_t ldda,
    magmaDoubleComplex_ptr dB, magma_int_t lddb,
    magma_queue_t queue);

magma_int_t
magma_izamax(
    magma_int_t n,
    magmaDoubleComplex_const_ptr dx, magma_int_t incx,
    magma_queue_t queue);

magma_int_t
magmablas_zaxpycp2(
    magma_int_t m,
    magmaDoubleComplex_ptr r, magmaDoubleComplex_ptr x, magmaDoubleComplex_ptr b,
    magma_queue_t queue);






#ifdef __cplusplus
}
#endif

#endif        //  #ifndef MAGMA_BLAS_H
