/*
 *   -- micMAGMA (version 1.3.1) --
 *      Univ. of Tennessee, Knoxville
 *      Univ. of California, Berkeley
 *      Univ. of Colorado, Denver
 *      @date January 2015
 *
 *   @author Azzam Haidar
 *   @author Khairul Kabir
 *   @author Mark Gates
 *   @author Hartwig Anzt
 *
 *   @generated from magmablas_z.cpp normal z -> d, Fri Jan 30 18:21:53 2015
 */

#include <stdio.h>
#include <stdlib.h>
#include "magma.h"
//#include "magmasparse.h"

#ifdef HAVE_MIC

#include "magma_mic_bindings.h"
// ========================================
// copying vectors
extern "C" magma_int_t
magma_dsetvector(
    magma_int_t n,
    double const* hx_src, magma_int_t incx,
    double*       dy_dst, magma_int_t incy,
    magma_queue_t queue )
{
    magma_event_t event;
    magma_mic_synchronize_kernels();
    magma_mic_memcpy2D_async(
        dy_dst, incy*sizeof(double),
        hx_src, incx*sizeof(double),
        sizeof(double), n,
        WRITETO_DEV, queue, &event );
    magma_mic_fence_wait( event );
    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_dgetvector(
   magma_int_t n,
   double const* dx_src, magma_int_t incx,
   double*       hy_dst, magma_int_t incy,
   magma_queue_t queue )
{
    magma_event_t event;
    magma_mic_synchronize_kernels();
    magma_mic_memcpy2D_async(
        hy_dst, incy*sizeof(double),
        dx_src, incx*sizeof(double),
        sizeof(double), n,
        READFROM_DEV, queue, &event );
    magma_mic_fence_wait( event );
    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_dsetvector_async(
    magma_int_t n,
    double const* hx_src, magma_int_t incx,
    double*       dy_dst, magma_int_t incy,
    magma_queue_t queue, magma_event_t *event )
{
    magma_mic_memcpy2D_async(
        dy_dst, incy*sizeof(double),
        hx_src, incx*sizeof(double),
        sizeof(double), n,
        WRITETO_DEV, queue, event );

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_dgetvector_async(
    magma_int_t m, magma_int_t n,
    magmaDouble_const_ptr dA_src, size_t dA_offset, magma_int_t ldda,
    double*          hA_dst, size_t hA_offset, magma_int_t ldha,
    magma_queue_t queue, magma_event_t *event )
{
    magma_mic_memcpy2D_async(
        hA_dst + hA_offset, ldha*sizeof(double),
        dA_src + dA_offset, ldda*sizeof(double),
        m*sizeof(double), n,
        READFROM_DEV, queue, event );
    return MAGMA_SUCCESS;
}


// ========================================
// copying sub-matrices (contiguous columns)
extern "C" magma_int_t
magma_dsetmatrix(
    magma_int_t m, magma_int_t n,
    double const* hA_src, size_t hA_offset, magma_int_t ldha,
    magmaDouble_ptr    dA_dst, size_t dA_offset, magma_int_t ldda,
    magma_queue_t queue )
{
    magma_event_t event;
    magma_mic_synchronize_kernels();
    magma_mic_memcpy2D_async(
        dA_dst + dA_offset, ldda*sizeof(double),
        hA_src + hA_offset, ldha*sizeof(double),
        m*sizeof(double), n,
        WRITETO_DEV, queue, &event );
    magma_mic_fence_wait( event );
    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_dgetmatrix(
    magma_int_t m, magma_int_t n,
    magmaDouble_const_ptr dA_src, size_t dA_offset, magma_int_t ldda,
    double*          hA_dst, size_t hA_offset, magma_int_t ldha,
    magma_queue_t queue )
{
    magma_event_t event;
    magma_mic_synchronize_kernels();
    magma_mic_memcpy2D_async(
        hA_dst + hA_offset, ldha*sizeof(double),
        dA_src + dA_offset, ldda*sizeof(double),
        m*sizeof(double), n,
        READFROM_DEV, queue, &event );
    magma_mic_fence_wait( event );
    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_dsetmatrix_async(
    magma_int_t m, magma_int_t n,
    double const* hA_src, size_t hA_offset, magma_int_t ldha,
    magmaDouble_ptr    dA_dst, size_t dA_offset, magma_int_t ldda,
    magma_queue_t queue, magma_event_t *event )
{
    if ( m < 1 || n < 1)
        return MAGMA_SUCCESS;

    magma_mic_memcpy2D_async(
        dA_dst + dA_offset, ldda*sizeof(double),
        hA_src + hA_offset, ldha*sizeof(double),
        m*sizeof(double), n,
        WRITETO_DEV, queue, event );
    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_dgetmatrix_async(
    magma_int_t m, magma_int_t n,
    magmaDouble_const_ptr dA_src, size_t dA_offset, magma_int_t ldda,
    double*          hA_dst, size_t hA_offset, magma_int_t ldha,
    magma_queue_t queue, magma_event_t *event )
{
    if ( m < 1 || n < 1) 
        return MAGMA_SUCCESS;

    magma_mic_memcpy2D_async(
        hA_dst + hA_offset, ldha*sizeof(double),
        dA_src + dA_offset, ldda*sizeof(double),
        m*sizeof(double), n,
        READFROM_DEV, queue, event );
    return MAGMA_SUCCESS;
}

// --------------------
// TODO this doesn't match naming scheme; use magma_isetvector and magma_igetvector
extern "C" magma_int_t
magma_dsetintarray_async(
    magma_int_t lda, magma_int_t const* hA_src, size_t hA_offset,
    magma_int_t *dA_dst, size_t dA_offset, magma_queue_t queue, magma_event_t *event )
{
    magma_int_t ncol = 1;
    magma_mic_memcpy2D_async(
        (void *)(dA_dst + dA_offset), lda * sizeof(magma_int_t),
        (void *)(hA_src + hA_offset), lda * sizeof(magma_int_t),
        lda * sizeof(magma_int_t), ncol,
        WRITETO_DEV, queue, event);
    return MAGMA_SUCCESS;
}

extern "C" magma_int_t
magma_dgetintarray_async(
    magma_int_t lda, magma_int_t const* hA_src, size_t hA_offset,
    magma_int_t *dA_dst, size_t dA_offset, magma_queue_t queue, magma_event_t *event )
{
    magma_mic_memcpy2D_async(
        (void *)(dA_dst + dA_offset), lda * sizeof(magma_int_t),
        (void *)(hA_src + hA_offset), lda * sizeof(magma_int_t),
        lda * sizeof(magma_int_t), 1,
        READFROM_DEV, queue, event);
    return MAGMA_SUCCESS;
}



extern "C" magma_int_t
magma_dprbt(
    magma_int_t n, 
    magmaDouble_ptr dA, magma_int_t ldda, 
    magmaDouble_const_ptr du, magmaDouble_const_ptr dv, 
    magma_queue_t handle)
{
	int err;
	magma_mic_dprbt_param param;

	param.n = n;
	param.d_A = dA;
	param.lda = ldda;
	param.d_u = du;
	param.d_v = dv;
	int control_msg = magma_mic_DPRBT;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

extern "C" magma_int_t
magma_dprbt_mtv(
    magma_int_t n, 
    magmaDouble_const_ptr du, magmaDouble_ptr db, 
    magma_queue_t handle)
{
	int err;
	magma_mic_dprbt_mtv_param param;

	param.n = n;
	param.d_u = du;
	param.d_b = db;
	int control_msg = magma_mic_DPRBT_MTV;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

	
extern "C" magma_int_t
magma_dprbt_mv(
    magma_int_t n, 
    magmaDouble_const_ptr dv, magmaDouble_ptr db, 
    magma_queue_t handle)
{
	int err;
	magma_mic_dprbt_mv_param param;

	param.n = n;
	param.d_v = dv;
	param.d_b = db;
	int control_msg = magma_mic_DPRBT_MV;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}




// ========================================
// BLAS functions
// --------------------


extern "C" magma_int_t
magma_dcsrgemv(
    magma_trans_t  trans,
    magma_int_t n,
    magmaDouble_const_ptr daval,
    magmaIndex_const_ptr darow,
    magmaIndex_const_ptr dacol,
    magmaDouble_const_ptr dx,
    magmaDouble_ptr dy,
    magma_queue_t handle )
{
    int err;
    magma_mic_dcsrgemv_param param;

    param.transa = trans;
    param.m      = n;
    param.a      = daval;
    param.ia     = darow;
    param.ja     = dacol;   
    param.x      = dx + 0;
    param.y      = dy + 0;

    int control_msg = magma_mic_DCSRGEMV;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}


extern "C" magma_int_t
magma_dcsrgespmm(
    magma_trans_t  trans,
    magma_int_t request,
    magma_int_t sort,
    magma_int_t m,
    magma_int_t n,
    magma_int_t k,
    double alpha,
    magmaDouble_const_ptr daval,
    magmaIndex_const_ptr darow,
    magmaIndex_const_ptr dacol,
    magmaDouble_const_ptr dbval,
    magmaIndex_const_ptr dbrow,
    magmaIndex_const_ptr dbcol,
    magmaDouble_ptr dcval,
    magmaIndex_ptr dcrow,
    magmaIndex_ptr dccol,
    magma_int_t nzmax,
    magma_int_t info,
    magma_queue_t handle )
{
    int err;
    magma_mic_dcsrgespmm_param param;

    param.transa = trans;
    param.request= request;
    param.sort   = sort;
    param.m      = m;
    param.n      = n;
    param.k      = k;
    param.alpha  = alpha;
    param.a      = daval;
    param.ia     = darow;
    param.ja     = dacol;  
    param.b      = dbval;
    param.ib     = dbrow;
    param.jb     = dbcol;  
    param.c      = dcval;
    param.ic     = dcrow;
    param.jc     = dccol;  
    param.nzmax  = nzmax;
    param.info   = info;
    
    int control_msg = magma_mic_DCSRGESPMM;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}


extern "C" double
magma_ddot(
    magma_int_t n,
    magmaDouble_const_ptr dx,
    magma_int_t incx,
    magmaDouble_const_ptr dy,
    magma_int_t incy,
    magma_queue_t handle )
{
    int err;
    magma_mic_ddot_param param;

    magmaDouble_ptr hres, dres;
    double result;
    magma_dmalloc_pinned(&hres, 1);
    magma_dmalloc(&dres, 1);

    param.res    = dres; 
    param.n      = n;
    param.x      = dx + 0;
    param.incx   = incx;
    param.y      = dy + 0;
    param.incy    = incy;

    int control_msg = magma_mic_DDOT;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    magma_mic_synchronize_kernels();
    magma_dgetvector(1, dres, 0, hres, 0, handle);

    result = *hres;

    free(hres);
    magma_free(dres);
    return result;
}


extern "C" double
magma_dnrm2(
    magma_int_t n,
    magmaDouble_const_ptr dx,
    magma_int_t incx,
    magma_queue_t handle )
{
    int err;
    magma_mic_dnrm2_param param;

    double *hres, *dres;
    double result;
    magma_dmalloc_pinned(&hres, 1);
    magma_dmalloc(&dres, 1);

    param.res    = dres;
    param.n      = n;
    param.x      = dx + 0;
    param.incx   = incx;

    int control_msg = magma_mic_DNRM2;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }
    magma_mic_synchronize_kernels();
    magma_dgetvector(1, dres, 0, hres, 0, handle);

    result = *hres;

    free(hres);
    magma_free(dres);
    return result;
}


extern "C" magma_int_t
magma_daxpy(
    magma_int_t n,
    double alpha,
    magmaDouble_const_ptr dx,
    magma_int_t incx,
    magmaDouble_ptr dy,
    magma_int_t incy,
    magma_queue_t handle )
{
    int err;
    magma_mic_daxpy_param param;

    param.n      = n;
    param.a      = alpha;
    param.x      = dx + 0;
    param.incx   = incx;
    param.y      = dy + 0;
    param.incy    = incy;

    int control_msg = magma_mic_DAXPY;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}



extern "C" magma_int_t
magma_dcopy(
    magma_int_t n,
    magmaDouble_const_ptr dx, magma_int_t incx,
    magmaDouble_ptr dy, magma_int_t incy,
    magma_queue_t handle )
{
    int err;
    magma_mic_dcopy_param param;

    param.n      = n;
    param.x      = dx;
    param.incx   = incx;
    param.y      = dy;
    param.incy    = incy;

    int control_msg = magma_mic_DCOPY;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_dcopyvector(
    magma_int_t n,
    magmaDouble_const_ptr dx_src, magma_int_t incx,
    magmaDouble_ptr dy_dst, magma_int_t incy,
    magma_queue_t handle )
{
    return magma_dcopy(n, dx_src, incx, dy_dst, incy, handle); 
}



// --------------------
extern "C" magma_int_t
magma_dgemm(
    magma_trans_t transA, magma_trans_t transB,
    magma_int_t m, magma_int_t n, magma_int_t k,
    double alpha, magmaDouble_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaDouble_const_ptr dB, size_t dB_offset, magma_int_t ldb,
    double beta,  magmaDouble_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_queue_t handle )
{
    int err;
    magma_mic_dgemm_param param;

    param.transa = transA;
    param.transb = transB;
    param.m      = m;
    param.n      = n;
    param.k      = k;
    param.alpha  = alpha;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.b      = dB + dB_offset;
    param.ldb    = ldb;
    param.beta   = beta;
    param.c      = dC + dC_offset;
    param.ldc    = ldc;

    int control_msg = magma_mic_DGEMM;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_dgemv(
    magma_trans_t transA,
    magma_int_t m, magma_int_t n,
    double alpha, magmaDouble_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaDouble_const_ptr dx, size_t dx_offset, magma_int_t incx,
    double beta,  magmaDouble_ptr       dy, size_t dy_offset, magma_int_t incy,
    magma_queue_t handle )
{
    int err;
    magma_mic_dgemv_param param;

    param.transa = transA;
    param.m      = m;
    param.n      = n;
    param.alpha  = alpha;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.x      = dx + dx_offset;
    param.incx   = incx;
    param.beta   = beta;
    param.y      = dy + dy_offset;
    param.incy    = incy;

    int control_msg = magma_mic_DGEMV;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_dsymm(
    magma_side_t side, magma_uplo_t uplo,
    magma_int_t m, magma_int_t n,
    double alpha, magmaDouble_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaDouble_const_ptr dB, size_t dB_offset, magma_int_t ldb,
    double beta,  magmaDouble_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_queue_t handle )
{
    int err;
    magma_mic_dsymm_param param;

    param.side   = side;
    param.uplo   = uplo;
    param.m      = m;
    param.n      = n;
    param.alpha  = alpha;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.b      = dB + dB_offset;
    param.ldb    = ldb;
    param.beta   = beta;
    param.c      = dC + dC_offset;
    param.ldc    = ldc;
    param.agboff = 0;

    int control_msg = magma_mic_DSYMM;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}
// --------------------
extern "C" magma_int_t
magma_dsymm_ours(
    magma_side_t side, magma_uplo_t uplo,
    magma_int_t m, magma_int_t n,
    double alpha, magmaDouble_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaDouble_const_ptr dB, size_t dB_offset, magma_int_t ldb,
    double beta,  magmaDouble_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_int_t globaldAoffset,
    magma_queue_t handle )
{
    int err;
    magma_mic_dsymm_param param;

    param.side   = side;
    param.uplo   = uplo;
    param.m      = m;
    param.n      = n;
    param.alpha  = alpha;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.b      = dB + dB_offset;
    param.ldb    = ldb;
    param.beta   = beta;
    param.c      = dC + dC_offset;
    param.ldc    = ldc;
    param.ldc    = ldc;
    param.agboff = globaldAoffset;

    int control_msg = magma_mic_DSYMM;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_dscal(
    magma_int_t n,
    double alpha,
    double *dx, magma_int_t incx,
    magma_queue_t handle )
{
    int err;
    magma_mic_dscal_param param = {n, alpha, dx, incx};

    int control_msg = magma_mic_DSCAL;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}


// --------------------
extern "C" magma_int_t
magma_dsymv2(
    magma_uplo_t uplo,
    magma_int_t n,
    double alpha, magmaDouble_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaDouble_const_ptr dx, size_t dx_offset, magma_int_t incx,
    double beta,  magmaDouble_ptr       dy, size_t dy_offset, magma_int_t incy,
    magma_queue_t handle )
{
    int err;
    magma_mic_dsymv2_param param;

    param.uplo   = uplo;
    param.n      = n;
    param.alpha  = alpha;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.x      = dx + dx_offset;
    param.incx   = incx;
    param.beta   = beta;
    param.y      = dy + dy_offset;
    param.incy   = incy;

    int control_msg = magma_mic_DSYMV2;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_dsymv(
    magma_uplo_t uplo,
    magma_int_t n,
    double alpha, magmaDouble_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaDouble_const_ptr dx, size_t dx_offset, magma_int_t incx,
    double beta,  magmaDouble_ptr       dy, size_t dy_offset, magma_int_t incy,
    magma_queue_t handle )
{
    int err;
    magma_mic_dsymv_param param;

    param.uplo   = uplo;
    param.n      = n;
    param.alpha  = alpha;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.x      = dx + dx_offset;
    param.incx   = incx;
    param.beta   = beta;
    param.y      = dy + dy_offset;
    param.incy    = incy;

    int control_msg = magma_mic_DSYMV;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_dsyrk(
    magma_uplo_t uplo,
    magma_trans_t trans,
    magma_int_t n, magma_int_t k,
    double alpha, magmaDouble_const_ptr dA, size_t dA_offset, magma_int_t lda,
    double beta,  magmaDouble_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_queue_t handle )
{
    int err;
    magma_mic_dsyrk_param param;

    param.uplo  = uplo;
    param.trans = trans;
    param.n     = n;
    param.k     = k;
    param.alpha = alpha;
    param.a     = dA + dA_offset;
    param.lda   = lda;
    param.beta  = beta;
    param.c     = dC + dC_offset;
    param.ldc   = ldc;

    int control_msg = magma_mic_DSYRK;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}


// --------------------
extern "C" magma_int_t
magma_dsyrk_mmic(
    magma_uplo_t uplo, magma_trans_t trans,
    magma_int_t n, magma_int_t k,
    double alpha, magmaDouble_const_ptr dA, magma_int_t lda, magma_int_t dAoff,
    double beta,  magmaDouble_const_ptr dC, magma_int_t ldc, magma_int_t offset,
    magma_int_t nmic, magma_int_t nb, magma_int_t my_id )
{
    int err;
    magma_mic_dsyrk_mmic_param param;

    param.uplo   = uplo;
    param.trans  = trans;
    param.n      = n;
    param.k      = k;
    param.alpha  = alpha;
    param.a      = dA;
    param.lda    = lda;
    param.aoff   = dAoff;
    param.beta   = beta;
    param.c      = dC;
    param.ldc    = ldc;
    param.offset = offset;
    param.nb     = nb;
    param.nmic   = nmic;
    param.my_id  = my_id;

    int control_msg = magma_mic_DSYRK_MMIC;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}



// --------------------
extern "C" magma_int_t
magma_dsyr2k(
    magma_uplo_t uplo, magma_trans_t trans,
    magma_int_t n, magma_int_t k,
    double alpha, magmaDouble_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaDouble_const_ptr dB, size_t dB_offset, magma_int_t ldb,
    double              beta, magmaDouble_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_queue_t queue )
{
    int err;
    magma_mic_dsyr2k_param param;

    param.uplo  = uplo;
    param.trans = trans;
    param.n     = n;
    param.k     = k;
    param.alpha = alpha;
    param.a     = dA + dA_offset;
    param.lda   = lda;
    param.b     = dB + dB_offset;
    param.ldb   = ldb;
    param.beta  = beta;
    param.c     = dC + dC_offset;
    param.ldc   = ldc;

    int control_msg = magma_mic_DSYR2K;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_dsyr2k_mmic(
    magma_uplo_t uplo, magma_trans_t trans, magma_int_t n, magma_int_t k,
    double alpha, magmaDouble_const_ptr dA, magma_int_t lda, magma_int_t dAoff,
                              magmaDouble_const_ptr dB, magma_int_t ldb, magma_int_t dBoff,
    double beta,              magmaDouble_const_ptr dC, magma_int_t ldc,  magma_int_t offset,
    magma_int_t nmic, magma_int_t nb, magma_int_t my_id)
{
    int err;
    magma_mic_dsyr2k_mmic_param param;

    param.uplo   = uplo;
    param.trans  = trans;
    param.n      = n;
    param.k      = k;
    param.alpha  = alpha;
    param.a      = dA;
    param.lda    = lda;
    param.aoff   = dAoff;
    param.b      = dB;
    param.ldb    = ldb;
    param.boff   = dBoff;
    param.beta   = beta;
    param.c      = dC;
    param.ldc    = ldc;
    param.offset = offset;
    param.nb     = nb;
    param.nmic   = nmic;
    param.my_id  = my_id;

    int control_msg = magma_mic_DSYR2K_MMIC;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}


// --------------------
extern "C" magma_int_t
magma_dtrsm(
    magma_side_t  side,
    magma_uplo_t  uplo,
    magma_trans_t trans,
    magma_diag_t  diag,
    magma_int_t m, magma_int_t n,
    double alpha, magmaDouble_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaDouble_ptr       dB, size_t dB_offset, magma_int_t ldb,
    magma_queue_t handle )
{
    int err;
    magma_mic_dtrsm_param param;

    param.side   = side;
    param.uplo   = uplo;
    param.transa = trans;
    param.diag   = diag;
    param.m      = m;
    param.n      = n;
    param.alpha  = alpha;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.b      = dB + dB_offset;
    param.ldb    = ldb;

    int control_msg = magma_mic_DTRSM;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_dtrsv(magma_uplo_t uplo, magma_trans_t trans, magma_diag_t diag,
    magma_int_t n,
    magmaDouble_const_ptr dA, size_t dA_offset, magma_int_t lda,
    magmaDouble_ptr dx, size_t dx_offset, magma_int_t incx,
    magma_queue_t queue )
{
    int err;
    magma_mic_dtrsv_param param;
    
    param.uplo   = uplo;
    param.trans  = trans;
    param.diag   = diag;
    param.n      = n;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.dx     = dx + dx_offset;
    param.incx   = incx;
    
    int control_msg = magma_mic_DTRSV;
    
    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }
    
    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }
    
    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_dtrmm(
    magma_side_t side, magma_uplo_t uplo, magma_trans_t trans, magma_diag_t diag,
    magma_int_t m, magma_int_t n,
    double alpha, magmaDouble_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaDouble_ptr       dB, size_t dB_offset, magma_int_t ldb,
    magma_queue_t queue )
{
    int err;
    magma_mic_dtrmm_param param;

    param.side   = side;
    param.uplo   = uplo;
    param.transa = trans;
    param.diag   = diag;
    param.m      = m;
    param.n      = n;
    param.alpha  = alpha;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.b      = dB + dB_offset;
    param.ldb    = ldb;

    int control_msg = magma_mic_DTRMM;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_dtrmv(magma_uplo_t uplo, magma_trans_t trans, magma_diag_t diag,
    magma_int_t n,
    magmaDouble_const_ptr dA, size_t dA_offset, magma_int_t lda,
    magmaDouble_ptr dx, size_t dx_offset, magma_int_t incx,
    magma_queue_t queue )
{
    int err;
    magma_mic_dtrmv_param param;
    
    param.uplo   = uplo;
    param.trans  = trans;
    param.diag   = diag;
    param.n      = n;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.dx     = dx + dx_offset;
    param.incx   = incx;
    
    int control_msg = magma_mic_DTRMV;
    
    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }
    
    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }
    
    return MAGMA_SUCCESS;
}




// ========================================
// Auxuliary routines
extern "C" magma_int_t
magma_dtranspose(magmaDouble_ptr odata, magma_int_t offo, magma_int_t ldo,
    magmaDouble_ptr idata, magma_int_t offi, magma_int_t ldi,
    magma_int_t m, magma_int_t n,
    magma_queue_t queue)
{
    int err;
    magma_mic_dtranspose_param param;

    param.odata = odata + offo;
    param.ldo   = ldo;
    param.idata = idata + offi;
    param.ldi   = ldi;
    param.m     = m;
    param.n     = n;

    int control_msg = magma_mic_DTRANSPOSE;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

extern "C" magma_int_t
magma_dlaswp(
    magma_int_t n, magmaDouble_ptr d_A, magma_int_t lda,
    magma_int_t k1, magma_int_t k2, magma_int_t *d_ipiv, magma_int_t incx,
    magma_queue_t queue)
{
    magma_event_t event;
    int err;
    magma_mic_dlaswp_param param;
    param.n      = n;
    param.d_A    = d_A;
    param.lda    = lda;
    param.k1     = k1;
    param.k2     = k2;
    param.d_ipiv = d_ipiv;
    param.incx   = incx;

    int control_msg = magma_mic_DLASWP;
    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }
    return MAGMA_SUCCESS;
}

extern "C" magma_int_t
magma_dswap(
    magma_int_t n, double *dA1T, magma_int_t lda1,
    double *dA2T, magma_int_t lda2,
    magma_queue_t queue)
{
    magma_event_t event;
    int err;
    magma_mic_dswap_param param;
    param.n      = n;
    param.dA1T    = dA1T;
    param.lda1    = lda1;
    param.dA2T    = dA2T;
    param.lda2    = lda2;

    int control_msg = magma_mic_DSWAP;
    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }
    return MAGMA_SUCCESS;
}

extern "C" magma_int_t
magma_dlasyf_mic(
    magma_uplo_t uplo, magma_int_t n, magma_int_t nb, magma_int_t *kb,
    double *hA, magma_int_t lda,
    magmaDouble_ptr dA, size_t dA_offset, magma_int_t ldda,
    magma_int_t *d_ipiv,
    magmaDouble_ptr dW, size_t dW_offset, magma_int_t lddw,
    magma_int_t *info, magma_queue_t queue )
{
    int err;
    magma_mic_dlasyf_param param = {uplo, n, nb, kb, hA, lda,
				    dA+dA_offset, ldda, d_ipiv, dW+dW_offset, lddw};

    int control_msg = magma_mic_DLASYF;
    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
       err = errno;
       printf("scif_send failed with err %d\n", errno);
       fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
       err = errno;
       printf("scif_send failed with err %d\n", errno);
       fflush(stdout);
    }
    return MAGMA_SUCCESS;
}

// This version sends a larfb request to the MIC. The difference with the version in
// 'src' is that here there is only one request and control is passed immediately back
// to the CPU.
extern "C" magma_int_t
magma_dlarfb(
    magma_side_t side, magma_trans_t trans, magma_direct_t direct, magma_storev_t storev,
    magma_int_t m, magma_int_t n, magma_int_t k,
    magmaDouble_ptr dV, size_t dV_offset, magma_int_t ldv,
    magmaDouble_ptr dT, size_t dT_offset, magma_int_t ldt,
    magmaDouble_ptr dC, size_t dC_offset, magma_int_t ldc,
    magmaDouble_ptr dwork, size_t dwork_offset, magma_int_t ldwork,
    magma_queue_t queue)
{
    int err;
    magma_mic_dlarfb_param param;

    param.side   = side;
    param.trans  = trans;
    param.direct = direct;
    param.storev = storev;
    param.m      = m;
    param.n      = n;
    param.k      = k;
    param.v      = dV + dV_offset;
    param.ldv    = ldv;
    param.t      = dT + dT_offset;
    param.ldt    = ldt;
    param.c      = dC + dC_offset;
    param.ldc    = ldc;
    param.work   = dwork + dwork_offset;
    param.ldwork = ldwork;

    int control_msg = magma_mic_DLARFB;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

extern "C" magma_int_t
magma_dlarfb_gemm(
    magma_side_t side, magma_trans_t trans, magma_direct_t direct, magma_storev_t storev,
    magma_int_t m, magma_int_t n, magma_int_t k,
    magmaDouble_ptr dV, size_t dV_offset, magma_int_t ldv,
    magmaDouble_ptr dT, size_t dT_offset, magma_int_t ldt,
    magmaDouble_ptr dC, size_t dC_offset, magma_int_t ldc,
    magmaDouble_ptr dwork, size_t dwork_offset, magma_int_t ldwork,
    magmaDouble_ptr dworkvt, size_t dworkvt_offset, magma_int_t ldworkvt,
    magma_queue_t queue)
{
    int err;
    magma_mic_dlarfb_gemm_param larfb_gemm_param;

    larfb_gemm_param.side   = side;
    larfb_gemm_param.trans  = trans;
    larfb_gemm_param.direct = direct;
    larfb_gemm_param.storev = storev;
    larfb_gemm_param.m      = m;
    larfb_gemm_param.n      = n;
    larfb_gemm_param.k      = k;
    larfb_gemm_param.v      = dV + dV_offset;
    larfb_gemm_param.ldv    = ldv;
    larfb_gemm_param.t      = dT + dT_offset;
    larfb_gemm_param.ldt    = ldt;
    larfb_gemm_param.c      = dC + dC_offset;
    larfb_gemm_param.ldc    = ldc;
    larfb_gemm_param.work   = dwork + dwork_offset;
    larfb_gemm_param.ldwork = ldwork;
    larfb_gemm_param.workvt   = dworkvt + dworkvt_offset;
    larfb_gemm_param.ldworkvt = ldworkvt;

    int control_msg = magma_mic_DLARFB_GEMM;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &larfb_gemm_param, sizeof(larfb_gemm_param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

extern "C" magma_int_t
magma_dlaset(magma_uplo_t uplo,
             magma_int_t m, magma_int_t n,
             double alpha, double beta,
             magmaDouble_ptr dA, size_t dA_offset, magma_int_t lda,
             magma_queue_t queue)
{
     int err;
     magma_mic_dlaset_param param;
     int control_msg = magma_mic_DLASET;

     param.uplo   = uplo;
     param.m      = m;
     param.n      = n;
     param.alpha  = alpha;
     param.beta   = beta;
     param.dA     = dA + dA_offset;
     param.lda    = lda;

     if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
         err = errno;
         printf("scif_send failed with err %d\n", errno);
         fflush(stdout);
     }

     if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
     }

     return MAGMA_SUCCESS;
}


// ========================================
// BLAS functions - combined blas for MIC
extern "C" magma_int_t
magma_dtrsmgemm(
    magma_side_t side, magma_uplo_t uplo,
    magma_trans_t trans, magma_diag_t diag,
    magma_int_t tm, magma_int_t tn,
    double talpha, magmaDouble_const_ptr tdA, size_t tdA_offset, magma_int_t tlda,
                               magmaDouble_ptr       tdB, size_t tdB_offset, magma_int_t tldb,
    
    magma_trans_t transA, magma_trans_t transB,
    magma_int_t m, magma_int_t n, magma_int_t k,
    double alpha, magmaDouble_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaDouble_const_ptr dB, size_t dB_offset, magma_int_t ldb,
    double beta,  magmaDouble_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_queue_t handle )
{
    int err;
    magma_mic_dtrsmgemm_param param;

    //trsm parameter
    param.trsm_param.side   = side;
    param.trsm_param.uplo   = uplo;
    param.trsm_param.transa = trans;
    param.trsm_param.diag   = diag;
    param.trsm_param.m      = tm;
    param.trsm_param.n      = tn;
    param.trsm_param.alpha  = talpha;
    param.trsm_param.a      = tdA + tdA_offset;
    param.trsm_param.lda    = tlda;
    param.trsm_param.b      = tdB + tdB_offset;
    param.trsm_param.ldb    = tldb;

    //gemm parameter
    param.gemm_param.transa = transA;
    param.gemm_param.transb = transB;
    param.gemm_param.m      = m;
    param.gemm_param.n      = n;
    param.gemm_param.k      = k;
    param.gemm_param.alpha  = alpha;
    param.gemm_param.a      = dA + dA_offset;
    param.gemm_param.lda    = lda;
    param.gemm_param.b      = dB + dB_offset;
    param.gemm_param.ldb    = ldb;
    param.gemm_param.beta   = beta;
    param.gemm_param.c      = dC + dC_offset;
    param.gemm_param.ldc    = ldc;

    int control_msg = magma_mic_DTRSM_GEMM;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

extern "C" double
magmablas_dlange(
    magma_norm_t norm, magma_int_t m, magma_int_t n,
    magmaDouble_const_ptr dA, magma_int_t ldda,
    magmaDouble_ptr dwork,
    magma_queue_t handle)
{
    int err;
    magma_mic_dlange_param param;
    
    double *ret;
    double ret2;
    double *dret;
    
    magma_dmalloc_pinned(&ret, 1);
    magma_dmalloc(&dret, 1);
  
    magma_event_t event; 

    param.norm   = norm ;   
    param.m      = m    ;    
    param.n      = n    ;    
    param.dA     = dA   ;   
    param.ldda   = ldda ;      
    param.dwork  = dwork;  
    param.ret    = dret;
    
    int control_msg = magma_mic_DLANGE;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

#define PRECISION_d
#if defined PRECISION_z | defined PRECISION_d

    

    
   
    magma_dgetvector(1, dret, 0, ret, 0, handle);
#else

   magma_sgetvector(1, dret, 0, ret, 0, handle);
#endif
    ret2 = *ret;
    free(ret);
    magma_free(dret);
    return ret2;
}

// --------------------

extern "C" magma_int_t
magmablas_dlacpy(
    magma_uplo_t uplo, magma_int_t m, magma_int_t n,
    magmaDouble_const_ptr dA, magma_int_t ldda,
    magmaDouble_ptr dB, magma_int_t lddb,
    magma_queue_t handle)
{
    int err;
    magma_mic_dlacpy_param param;
    
    
    param.uplo   = uplo ;   
    param.m      = m    ;    
    param.n      = n    ;    
    param.dA     = dA   ;   
    param.ldda   = ldda ;      
    param.dB     = dB   ;   
    param.lddb   = lddb ;      
    
    int control_msg = magma_mic_DLACPY;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magmablas_dlascl_diag(
    magma_uplo_t uplo, magma_int_t m, magma_int_t n,
    magmaDouble_ptr dA, magma_int_t ldda,
    magmaDouble_ptr dB, magma_int_t lddb,
    magma_queue_t handle)
{
    int err;
    magma_mic_dlascl_diag_param param;

    param.uplo   = uplo ;
    param.m      = m    ;
    param.n      = n    ;
    param.dA     = dA   ;
    param.ldda   = ldda ;
    param.dB     = dB   ;
    param.lddb   = lddb ;

    int control_msg = magma_mic_DLASCL_DIAG;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------

extern "C" magma_int_t
magma_idamax(
    magma_int_t n,
    magmaDouble_const_ptr dx, magma_int_t incx,
    magma_queue_t handle)
{
    int err;
    magma_mic_idamax_param param;
   
    magma_int_t *ret;
    magma_int_t ret2;
    magma_int_t *dret;
     
    magma_imalloc_pinned(&ret, 1);
    magma_imalloc(&dret, 1);
  
    magma_event_t event; 

    
    param.n      = n    ;    
    param.dx     = dx   ;   
    param.incx   = incx ;      
    param.ret   = dret ;      
    
    int control_msg = magma_mic_IDAMAX;



    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }
    magma_mic_synchronize_kernels();




    magma_dgetintarray_async(1, dret, 0, ret, 0, handle, &event);
    
    magma_event_sync(event);
    ret2 = *ret;

    free(ret);
    magma_free(dret);
    return ret2;
}

extern "C" magma_int_t
magmablas_daxpycp2(
    magma_int_t m,
    magmaDouble_ptr r, magmaDouble_ptr x,magmaDouble_ptr b,
    magma_queue_t handle)
{
    int err;
    magma_mic_daxpycp2_param param;
    
    
    param.m      = m   ;    
    param.r      = r   ;   
    param.x      = x   ;   
    param.b      = b   ;   
    
    int control_msg = magma_mic_DAXPYCP2;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}



// SPARSE MAGMA
// ========================================
// Jacobi precond application
extern "C" magma_int_t
magma_djacobidiagscale(
    magma_int_t n,
    magma_d_vector d,
    magma_d_vector b,
    magma_d_vector *c,
    magma_queue_t queue )
{
    int err;
    magma_mic_djacobidiagscale_param param;

    param.m      = n;
    param.n      = b.num_rows*b.num_cols/n;
    param.b      = b.dval;
    param.d      = d.dval;
    param.c      = c->dval;
    
    int control_msg = magma_mic_DJACOBIDIAGSCALE;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

#endif // HAVE_MIC
