/*
 *   -- micMAGMA (version 1.3.1) --
 *      Univ. of Tennessee, Knoxville
 *      Univ. of California, Berkeley
 *      Univ. of Colorado, Denver
 *      April 2012
 *
 *   @author Azzam Haidar
 *   @author Khairul Kabir
 *   @author Stan Tomov
 *   @author Hartwig Anzt
 *
 *   @generated from magma_z_kernels.h normal z -> c, Fri Jan 30 18:21:54 2015
 *
 *   This is included in magma_mic_server.cpp,
 *   inside a switch { } block in the do_work() function.
 */

#define PRECISION_c

case magma_mic_CCSRGEMV:
{
    magma_mic_ccsrgemv_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CCSRGEMV\n");
        return;
    }

    mic_time = dsecnd();
    //call_ccsrgemv(param);
    if (param.transa == MagmaNoTrans) {
		mkl_ccsrgemv("No Transpose", &param.m, (MKL_Complex8 *)param.a, 
                     (MKL_INT *)param.ia, (MKL_INT *)param.ja, (MKL_Complex8 *)param.x, (MKL_Complex8 *)param.y);
	}else{
        mkl_ccsrgemv("Transpose", &param.m, (MKL_Complex8 *)param.a,
                     (MKL_INT *)param.ia, (MKL_INT *)param.ja, (MKL_Complex8 *)param.x, (MKL_Complex8 *)param.y);
	}
	mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_CCSRGESPMM:
{
    magma_mic_ccsrgespmm_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZCSRGESPMM\n");
        return;
    }

    mic_time = dsecnd();
    //call_ccsrgemv(param);
    if (param.transa == MagmaNoTrans) {
        mkl_ccsrmultcsr("No Transpose", &param.request, &param.sort, &param.m, &param.n, &param.k, 
                        (MKL_Complex8 *)param.a, (MKL_INT *)param.ja, (MKL_INT *)param.ia, 
                        (MKL_Complex8 *)param.b, (MKL_INT *)param.jb, (MKL_INT *)param.ib, 
                        (MKL_Complex8 *)param.c, (MKL_INT *)param.jc, (MKL_INT *)param.ic, &param.nzmax, &param.info);
	}else{
        mkl_ccsrmultcsr("Transpose", &param.request, &param.sort, &param.m, &param.n, &param.k, 
                        (MKL_Complex8 *)param.a, (MKL_INT *)param.ja, (MKL_INT *)param.ia, 
                        (MKL_Complex8 *)param.b, (MKL_INT *)param.jb, (MKL_INT *)param.ib, 
                        (MKL_Complex8 *)param.c, (MKL_INT *)param.jc, (MKL_INT *)param.ic, &param.nzmax, &param.info);
	}
	mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_CDOTC:
{
    //printf("cdotc\n");
    magma_mic_cdotc_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CDOTC\n");
        return;
    }

    mic_time = dsecnd();
    #if defined (PRECISION_z ) || defined( PRECISION_c )
    cdotc((MKL_Complex8 *) &param.res,
          (const MKL_INT *) &param.n, 
          (const MKL_Complex8 *) param.x, (const MKL_INT *) &param.incx,
          (const MKL_Complex8 *) param.y, (const MKL_INT *) &param.incy);
    #else
    *param.res = cdotc((const MKL_INT *) &param.n,
                      (const MKL_Complex8 *) param.x, (const MKL_INT *) &param.incx,
                      (const MKL_Complex8 *) param.y, (const MKL_INT *) &param.incy);
    #endif
    mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_SCNRM2:
{
    //printf("scnrm2\n");
    magma_mic_scnrm2_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SCNRM2\n");
        return;
    }

    mic_time = dsecnd();
    *param.res = scnrm2((const MKL_INT *) &param.n, (const MKL_Complex8 *) param.x, 
                        (const MKL_INT *) &param.incx);
    mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_CAXPY:
{
    //printf("caxpy\n");
    magma_mic_caxpy_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CAXPY\n");
        return;
    }

    mic_time = dsecnd();
    //call_caxpy(param);
    caxpy( (const MKL_INT *) &param.n, (const MKL_Complex8 *) &param.a, 
        (const MKL_Complex8 *) param.x,  (const MKL_INT *) &param.incx,  
        (MKL_Complex8 *) param.y, (const MKL_INT *) &param.incy );
    mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_CCOPY:
{
    //printf("ccopy\n");
    magma_mic_ccopy_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CCOPY\n");
        return;
    }

    mic_time = dsecnd();
    call_ccopy(param);
    mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_CGEMM:
{
    //printf("cgemm\n");
    magma_mic_cgemm_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CGEMM\n");
        return;
    }

    mic_time = dsecnd();
    call_cgemm(param);
    mic_time = dsecnd() - mic_time;

    m = param.m;
    n = param.n;
    k = param.k;
    gflops = MICFLOPS_GEMM((float)m, (float)n, (float)k) * 1e-9;
    //printf("cgemm: %d\t%d\t%d\t%8.2lf\t%8.2lf\t%8.2lf\n", m, n, k, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_CHEMM:
{
    //printf("chemm\n");
    magma_mic_chemm_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CHEMM\n");
        return;
    }

    mic_time = dsecnd();
    call_chemm(param);
    mic_time = dsecnd() - mic_time;

    m = param.m;
    n = param.n;
    magma_side_t side = param.side;

    //gflops = MICFLOPS_SYMM((magma_side_t)side, (float)m, (float)n ) * 1e-9;
    //printf("chemm: %d\t%d\t%g\t%g\t%g\n", m, n, mic_time,gflops, gflops/mic_time);
    break;
}

case magma_mic_CGEMV:
{
    //printf("cgemv\n");
    magma_mic_cgemv_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CGEMV\n");
        return;
    }

    mic_time = dsecnd();
    call_cgemv(param);
    mic_time = dsecnd() - mic_time;

    m = param.m;
    n = param.n;
    //gflops = MICFLOPS_GEMV((float)m, (float)n) * 1e-9;
    //printf("cgemv: %d\t%d\t%g\t%g\t%g\n", m, n, mic_time,
    //        gflops, gflops/mic_time);
    break;
}

case magma_mic_CHEMV:
{
    //printf("chemv\n");
    magma_mic_chemv_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CHEMV\n");
        return;
    }

    mic_time = dsecnd();
    call_chemv(param);
    mic_time = dsecnd() - mic_time;

    break;
}

case magma_mic_CSYMV2:
{
    //printf("csymv2\n");
    magma_mic_csymv2_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CHEMV\n");
        return;
    }

    mic_time = dsecnd();
    call_csymv2(param);
    mic_time = dsecnd() - mic_time;

    break;
}

case magma_mic_CTRSM:
{
    //printf("ctrsm\n");
    magma_mic_ctrsm_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CTRSM\n");
        return;
    }

    mic_time = dsecnd();
    call_ctrsm(param);
    mic_time = dsecnd() - mic_time;

    side = param.side;
    m = param.m;
    n = param.n;

    if (side == MagmaLeft)
        gflops = MICFLOPS_TRSM_L((float)m, (float)n) * 1e-9;
    else
        gflops = MICFLOPS_TRSM_R((float)m, (float)n) * 1e-9;

    //printf("ctrsm: %c\t%d\t%d\t%f\t%f\t%f\n", side, m, n, mic_time,
    //     gflops, gflops/mic_time);
    break;
}

case magma_mic_CTRSV:
{
    //printf("ctrsv\n");

    magma_mic_ctrsv_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CTRSV\n");
        return;
    }
    
    mic_time = dsecnd();
    call_ctrsv(param);
    mic_time = dsecnd() - mic_time;
    
    break;
}

case magma_mic_CTRMV:
{
    //printf("ctrmv\n");
    
    magma_mic_ctrmv_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CTRMV\n");
        return;
    }
    
    mic_time = dsecnd();
    call_ctrmv(param);
    mic_time = dsecnd() - mic_time;
    
    break;
}

case magma_mic_CTRMM:
{
    //printf("ctrmm\n");
    magma_mic_ctrmm_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CTRMM\n");
        return;
    }

    mic_time = dsecnd();
    call_ctrmm(param);
    mic_time = dsecnd() - mic_time;

    side = param.side;
    m = param.m;
    n = param.n;

    //if (side == MagmaLeft)
    //    gflops = MICFLOPS_TRSM_L((float)m, (float)n) * 1e-9;
    //else
    //    gflops = MICFLOPS_TRSM_R((float)m, (float)n) * 1e-9;

    //printf("ctrmm: %c\t%d\t%d\t%f\t%f\t%f\n", side, m, n, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_CHERK:
{
    //printf("cherk\n");
    magma_mic_cherk_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CHERK\n");
        return;
    }

    mic_time = dsecnd();
    call_cherk(param);
    mic_time = dsecnd() - mic_time;

    n = param.n;
    k = param.k;
    //gflops = MICFLOPS_SYRK((float)n, (float)k) * 1e-9;
    //printf("cherk: %d\t%d\t%f\t%f\t%f\n", n, k, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_CHERK_MMIC:
{
    //printf("cherk\n");
    magma_mic_cherk_mmic_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CHERK_MMIC\n");
        return;
    }

    mic_time = dsecnd();
    call_cherk_mmic(param);
    mic_time = dsecnd() - mic_time;

    n = param.n;
    k = param.k;
    //gflops = MICFLOPS_SYRK((float)n, (float)k) * 1e-9;
    //printf("cherk: %d\t%d\t%f\t%f\t%f\n", n, k, mic_time, gflops, gflops/mic_time);
    break;
}


case magma_mic_CHER2K:
{
    //printf("cher2k\n");
    magma_mic_cher2k_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CHER2K\n");
        return;
    }

    mic_time = dsecnd();
    call_cher2k(param);
    mic_time = dsecnd() - mic_time;

    n = param.n;
    k = param.k;
    //gflops = MICFLOPS_SYR2K((float)n, (float)k) * 1e-9;
    //printf("cher2k: %d\t%d\t%f\t%f\t%f\n", n, k, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_CHER2K_MMIC:
{
    //printf("cher2k_mmic\n");
    magma_mic_cher2k_mmic_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CHER2K_MMIC\n");
        return;
    }

    //mic_time = dsecnd();
    call_cher2k_mmic(param);
    //mic_time = dsecnd() - mic_time;

    n = param.n;
    k = param.k;
    //gflops = MICFLOPS_SYRK((float)n, (float)k) * 1e-9;
    //printf("cherk: %d\t%d\t%f\t%f\t%f\n", n, k, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_CLARFT:
{
    //printf("clarft\n");
    magma_mic_clarft_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CLARFT\n");
        return;
    }

    //mic_time = dsecnd();
    call_clarft(param);
    //mic_time = dsecnd() - mic_time;

    //gflops = MICFLOPS_GEMM((float)m, (float)n, (float)k) * 1e-9;
    //printf("clarft: %d\t%d\t%d\t%f\t%f\t%f\n", m, n, k, mic_time,
    //     gflops, gflops/mic_time);
    break;
}

case magma_mic_CLARFB:
{
    //printf("clarfb\n");
    magma_mic_clarfb_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CLARFB\n");
        return;
    }

    //mic_time = dsecnd();
    call_clarfb(param);
    //mic_time = dsecnd() - mic_time;

    //gflops = MICFLOPS_GEMM((float)m, (float)n, (float)k) * 1e-9;
    //printf("clarfb: %d\t%d\t%d\t%f\t%f\t%f\n", m, n, k, mic_time,
    //     gflops, gflops/mic_time);
    break;
}

case magma_mic_CLARFB_GEMM:
{
    //printf("clarfb\n");
    magma_mic_clarfb_gemm_param clarfb_gemm_param;
    if (mic_recv(&clarfb_gemm_param, sizeof(clarfb_gemm_param)) == MIC_FAILURE) {
        printf("scif_recv failed for CLARFB_GEMM\n");
        return;
    }

    //mic_time = dsecnd();
    call_clarfb_gemm(clarfb_gemm_param);
    //mic_time = dsecnd() - mic_time;

    //gflops = MICFLOPS_GEMM((float)m, (float)n, (float)k) * 1e-9;
    //printf("clarfb: %d\t%d\t%d\t%f\t%f\t%f\n", m, n, k, mic_time,
    //     gflops, gflops/mic_time);
    break;
}

case magma_mic_CTRSM_GEMM:
{
    //printf("ctrsmgemm\n");
    magma_mic_ctrsmgemm_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CTRSM_GEMM\n");
        return;
    }

    mic_time = dsecnd();
    call_ctrsmgemm(param);
    mic_time = dsecnd() - mic_time;

    break;
}

case magma_mic_CLASET:
{
    magma_mic_claset_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CLASET\n");
        return;
    }

    call_claset(param);

    break;
}

case magma_mic_CLASWP:
{
    magma_mic_claswp_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CLASWP\n");
        return;
    }

    call_claswp(param);

    break;
}

case magma_mic_CPRBT:
{

	magma_mic_cprbt_param param;
//printf("je suis dans le bon cas\n");
	if(mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
		printf("scif_recv failed for CPRBT\n");
		return;
    }

	call_cprbt(param);

	break;
}

case magma_mic_CPRBT_MTV:
{

	magma_mic_cprbt_mtv_param param;
//printf("je suis dans le bon cas\n");
	if(mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
		printf("scif_recv failed for CPRBT_MTV\n");
		return;
    }

	call_cprbt_mtv(param);

	break;
}

case magma_mic_CPRBT_MV:
{

	magma_mic_cprbt_mv_param param;
//printf("je suis dans le bon cas\n");
	if(mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
		printf("scif_recv failed for CPRBT_MV\n");
		return;
    }

	call_cprbt_mv(param);

	break;
}


case magma_mic_CSWAP:
{
    magma_mic_cswap_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CSWAP\n");
        return;
    }

    call_cswap(param);

    break;
}

case magma_mic_CLAHEF:
{
    magma_mic_clahef_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
      printf("scif_recv failed for CLAHEF\n");
      return;
    }

    call_clahef(param);

    break;
}

case magma_mic_CTRANSPOSE:
{
    magma_mic_ctranspose_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CTRANSPOSE\n");
        return;
    }

    call_ctranspose(param);
    break;
}

case magma_mic_CLANGE:
{
    magma_mic_clange_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CLANGE\n");
        return;
    }

    call_clange(param);
    break;
}

case magma_mic_CLACPY:
{
    magma_mic_clacpy_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CLACPY\n");
        return;
    }

    call_clacpy(param);
    break;
}

case magma_mic_CAXPYCP2:
{
    magma_mic_caxpycp2_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CAXPYCP2\n");
        return;
    }

    call_caxpycp2(param);
    break;
}

case magma_mic_CSCAL:
{
    magma_mic_cscal_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CAXPYCP2\n");
        return;
    }

    cscal(&param.n, (const MKL_Complex8 *)&param.alpha, (MKL_Complex8 *)param.dx, &param.incx);
    break;
}

case magma_mic_CLASCL_DIAG:
{
    magma_mic_clascl_diag_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CLASCL_DIAG\n");
        return;
    }

    call_clascl_diag(param);
    break;

}

case magma_mic_ICAMAX:
{
    magma_mic_icamax_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ICAMAX\n");
        return;
    }

    call_icamax(param);
    break;
}


// SPARSE MAGMA


case magma_mic_CJACOBIDIAGSCALE:
{
    magma_mic_cjacobidiagscale_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZJACOBIDIAGSCALE\n");
        return;
    }

    call_cjacobidiagscale(param);
    break;
}



#undef PRECISION_c
