/*
 *   -- micMAGMA (version 1.3.1) --
 *      Univ. of Tennessee, Knoxville
 *      Univ. of California, Berkeley
 *      Univ. of Colorado, Denver
 *      April 2012
 *
 *   @author Azzam Haidar
 *   @author Khairul Kabir
 *   @author Stan Tomov
 *   @author Hartwig Anzt
 *
 *   @generated from magma_z_kernels.h normal z -> d, Fri Jan 30 18:21:54 2015
 *
 *   This is included in magma_mic_server.cpp,
 *   inside a switch { } block in the do_work() function.
 */

#define PRECISION_d

case magma_mic_DCSRGEMV:
{
    magma_mic_dcsrgemv_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DCSRGEMV\n");
        return;
    }

    mic_time = dsecnd();
    //call_dcsrgemv(param);
    if (param.transa == MagmaNoTrans) {
		mkl_dcsrgemv("No Transpose", &param.m, (double *)param.a, 
                     (MKL_INT *)param.ia, (MKL_INT *)param.ja, (double *)param.x, (double *)param.y);
	}else{
        mkl_dcsrgemv("Transpose", &param.m, (double *)param.a,
                     (MKL_INT *)param.ia, (MKL_INT *)param.ja, (double *)param.x, (double *)param.y);
	}
	mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_DCSRGESPMM:
{
    magma_mic_dcsrgespmm_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZCSRGESPMM\n");
        return;
    }

    mic_time = dsecnd();
    //call_dcsrgemv(param);
    if (param.transa == MagmaNoTrans) {
        mkl_dcsrmultcsr("No Transpose", &param.request, &param.sort, &param.m, &param.n, &param.k, 
                        (double *)param.a, (MKL_INT *)param.ja, (MKL_INT *)param.ia, 
                        (double *)param.b, (MKL_INT *)param.jb, (MKL_INT *)param.ib, 
                        (double *)param.c, (MKL_INT *)param.jc, (MKL_INT *)param.ic, &param.nzmax, &param.info);
	}else{
        mkl_dcsrmultcsr("Transpose", &param.request, &param.sort, &param.m, &param.n, &param.k, 
                        (double *)param.a, (MKL_INT *)param.ja, (MKL_INT *)param.ia, 
                        (double *)param.b, (MKL_INT *)param.jb, (MKL_INT *)param.ib, 
                        (double *)param.c, (MKL_INT *)param.jc, (MKL_INT *)param.ic, &param.nzmax, &param.info);
	}
	mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_DDOT:
{
    //printf("ddot\n");
    magma_mic_ddot_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DDOT\n");
        return;
    }

    mic_time = dsecnd();
    #if defined (PRECISION_z ) || defined( PRECISION_c )
    ddot((double *) &param.res,
          (const MKL_INT *) &param.n, 
          (const double *) param.x, (const MKL_INT *) &param.incx,
          (const double *) param.y, (const MKL_INT *) &param.incy);
    #else
    *param.res = ddot((const MKL_INT *) &param.n,
                      (const double *) param.x, (const MKL_INT *) &param.incx,
                      (const double *) param.y, (const MKL_INT *) &param.incy);
    #endif
    mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_DNRM2:
{
    //printf("dnrm2\n");
    magma_mic_dnrm2_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DNRM2\n");
        return;
    }

    mic_time = dsecnd();
    *param.res = dnrm2((const MKL_INT *) &param.n, (const double *) param.x, 
                        (const MKL_INT *) &param.incx);
    mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_DAXPY:
{
    //printf("daxpy\n");
    magma_mic_daxpy_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DAXPY\n");
        return;
    }

    mic_time = dsecnd();
    //call_daxpy(param);
    daxpy( (const MKL_INT *) &param.n, (const double *) &param.a, 
        (const double *) param.x,  (const MKL_INT *) &param.incx,  
        (double *) param.y, (const MKL_INT *) &param.incy );
    mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_DCOPY:
{
    //printf("dcopy\n");
    magma_mic_dcopy_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DCOPY\n");
        return;
    }

    mic_time = dsecnd();
    call_dcopy(param);
    mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_DGEMM:
{
    //printf("dgemm\n");
    magma_mic_dgemm_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DGEMM\n");
        return;
    }

    mic_time = dsecnd();
    call_dgemm(param);
    mic_time = dsecnd() - mic_time;

    m = param.m;
    n = param.n;
    k = param.k;
    gflops = MICFLOPS_GEMM((double)m, (double)n, (double)k) * 1e-9;
    //printf("dgemm: %d\t%d\t%d\t%8.2lf\t%8.2lf\t%8.2lf\n", m, n, k, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_DSYMM:
{
    //printf("dsymm\n");
    magma_mic_dsymm_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DSYMM\n");
        return;
    }

    mic_time = dsecnd();
    call_dsymm(param);
    mic_time = dsecnd() - mic_time;

    m = param.m;
    n = param.n;
    magma_side_t side = param.side;

    //gflops = MICFLOPS_SYMM((magma_side_t)side, (double)m, (double)n ) * 1e-9;
    //printf("dsymm: %d\t%d\t%g\t%g\t%g\n", m, n, mic_time,gflops, gflops/mic_time);
    break;
}

case magma_mic_DGEMV:
{
    //printf("dgemv\n");
    magma_mic_dgemv_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DGEMV\n");
        return;
    }

    mic_time = dsecnd();
    call_dgemv(param);
    mic_time = dsecnd() - mic_time;

    m = param.m;
    n = param.n;
    //gflops = MICFLOPS_GEMV((double)m, (double)n) * 1e-9;
    //printf("dgemv: %d\t%d\t%g\t%g\t%g\n", m, n, mic_time,
    //        gflops, gflops/mic_time);
    break;
}

case magma_mic_DSYMV:
{
    //printf("dsymv\n");
    magma_mic_dsymv_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DSYMV\n");
        return;
    }

    mic_time = dsecnd();
    call_dsymv(param);
    mic_time = dsecnd() - mic_time;

    break;
}

case magma_mic_DSYMV2:
{
    //printf("dsymv2\n");
    magma_mic_dsymv2_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DSYMV\n");
        return;
    }

    mic_time = dsecnd();
    call_dsymv2(param);
    mic_time = dsecnd() - mic_time;

    break;
}

case magma_mic_DTRSM:
{
    //printf("dtrsm\n");
    magma_mic_dtrsm_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DTRSM\n");
        return;
    }

    mic_time = dsecnd();
    call_dtrsm(param);
    mic_time = dsecnd() - mic_time;

    side = param.side;
    m = param.m;
    n = param.n;

    if (side == MagmaLeft)
        gflops = MICFLOPS_TRSM_L((double)m, (double)n) * 1e-9;
    else
        gflops = MICFLOPS_TRSM_R((double)m, (double)n) * 1e-9;

    //printf("dtrsm: %c\t%d\t%d\t%f\t%f\t%f\n", side, m, n, mic_time,
    //     gflops, gflops/mic_time);
    break;
}

case magma_mic_DTRSV:
{
    //printf("dtrsv\n");

    magma_mic_dtrsv_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DTRSV\n");
        return;
    }
    
    mic_time = dsecnd();
    call_dtrsv(param);
    mic_time = dsecnd() - mic_time;
    
    break;
}

case magma_mic_DTRMV:
{
    //printf("dtrmv\n");
    
    magma_mic_dtrmv_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DTRMV\n");
        return;
    }
    
    mic_time = dsecnd();
    call_dtrmv(param);
    mic_time = dsecnd() - mic_time;
    
    break;
}

case magma_mic_DTRMM:
{
    //printf("dtrmm\n");
    magma_mic_dtrmm_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DTRMM\n");
        return;
    }

    mic_time = dsecnd();
    call_dtrmm(param);
    mic_time = dsecnd() - mic_time;

    side = param.side;
    m = param.m;
    n = param.n;

    //if (side == MagmaLeft)
    //    gflops = MICFLOPS_TRSM_L((float)m, (float)n) * 1e-9;
    //else
    //    gflops = MICFLOPS_TRSM_R((float)m, (float)n) * 1e-9;

    //printf("dtrmm: %c\t%d\t%d\t%f\t%f\t%f\n", side, m, n, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_DSYRK:
{
    //printf("dsyrk\n");
    magma_mic_dsyrk_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DSYRK\n");
        return;
    }

    mic_time = dsecnd();
    call_dsyrk(param);
    mic_time = dsecnd() - mic_time;

    n = param.n;
    k = param.k;
    //gflops = MICFLOPS_SYRK((float)n, (float)k) * 1e-9;
    //printf("dsyrk: %d\t%d\t%f\t%f\t%f\n", n, k, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_DSYRK_MMIC:
{
    //printf("dsyrk\n");
    magma_mic_dsyrk_mmic_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DSYRK_MMIC\n");
        return;
    }

    mic_time = dsecnd();
    call_dsyrk_mmic(param);
    mic_time = dsecnd() - mic_time;

    n = param.n;
    k = param.k;
    //gflops = MICFLOPS_SYRK((float)n, (float)k) * 1e-9;
    //printf("dsyrk: %d\t%d\t%f\t%f\t%f\n", n, k, mic_time, gflops, gflops/mic_time);
    break;
}


case magma_mic_DSYR2K:
{
    //printf("dsyr2k\n");
    magma_mic_dsyr2k_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DSYR2K\n");
        return;
    }

    mic_time = dsecnd();
    call_dsyr2k(param);
    mic_time = dsecnd() - mic_time;

    n = param.n;
    k = param.k;
    //gflops = MICFLOPS_SYR2K((float)n, (float)k) * 1e-9;
    //printf("dsyr2k: %d\t%d\t%f\t%f\t%f\n", n, k, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_DSYR2K_MMIC:
{
    //printf("dsyr2k_mmic\n");
    magma_mic_dsyr2k_mmic_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DSYR2K_MMIC\n");
        return;
    }

    //mic_time = dsecnd();
    call_dsyr2k_mmic(param);
    //mic_time = dsecnd() - mic_time;

    n = param.n;
    k = param.k;
    //gflops = MICFLOPS_SYRK((float)n, (float)k) * 1e-9;
    //printf("dsyrk: %d\t%d\t%f\t%f\t%f\n", n, k, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_DLARFT:
{
    //printf("dlarft\n");
    magma_mic_dlarft_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DLARFT\n");
        return;
    }

    //mic_time = dsecnd();
    call_dlarft(param);
    //mic_time = dsecnd() - mic_time;

    //gflops = MICFLOPS_GEMM((float)m, (float)n, (float)k) * 1e-9;
    //printf("dlarft: %d\t%d\t%d\t%f\t%f\t%f\n", m, n, k, mic_time,
    //     gflops, gflops/mic_time);
    break;
}

case magma_mic_DLARFB:
{
    //printf("dlarfb\n");
    magma_mic_dlarfb_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DLARFB\n");
        return;
    }

    //mic_time = dsecnd();
    call_dlarfb(param);
    //mic_time = dsecnd() - mic_time;

    //gflops = MICFLOPS_GEMM((float)m, (float)n, (float)k) * 1e-9;
    //printf("dlarfb: %d\t%d\t%d\t%f\t%f\t%f\n", m, n, k, mic_time,
    //     gflops, gflops/mic_time);
    break;
}

case magma_mic_DLARFB_GEMM:
{
    //printf("dlarfb\n");
    magma_mic_dlarfb_gemm_param dlarfb_gemm_param;
    if (mic_recv(&dlarfb_gemm_param, sizeof(dlarfb_gemm_param)) == MIC_FAILURE) {
        printf("scif_recv failed for DLARFB_GEMM\n");
        return;
    }

    //mic_time = dsecnd();
    call_dlarfb_gemm(dlarfb_gemm_param);
    //mic_time = dsecnd() - mic_time;

    //gflops = MICFLOPS_GEMM((float)m, (float)n, (float)k) * 1e-9;
    //printf("dlarfb: %d\t%d\t%d\t%f\t%f\t%f\n", m, n, k, mic_time,
    //     gflops, gflops/mic_time);
    break;
}

case magma_mic_DTRSM_GEMM:
{
    //printf("dtrsmgemm\n");
    magma_mic_dtrsmgemm_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DTRSM_GEMM\n");
        return;
    }

    mic_time = dsecnd();
    call_dtrsmgemm(param);
    mic_time = dsecnd() - mic_time;

    break;
}

case magma_mic_DLASET:
{
    magma_mic_dlaset_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DLASET\n");
        return;
    }

    call_dlaset(param);

    break;
}

case magma_mic_DLASWP:
{
    magma_mic_dlaswp_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DLASWP\n");
        return;
    }

    call_dlaswp(param);

    break;
}

case magma_mic_DPRBT:
{

	magma_mic_dprbt_param param;
//printf("je suis dans le bon cas\n");
	if(mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
		printf("scif_recv failed for DPRBT\n");
		return;
    }

	call_dprbt(param);

	break;
}

case magma_mic_DPRBT_MTV:
{

	magma_mic_dprbt_mtv_param param;
//printf("je suis dans le bon cas\n");
	if(mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
		printf("scif_recv failed for DPRBT_MTV\n");
		return;
    }

	call_dprbt_mtv(param);

	break;
}

case magma_mic_DPRBT_MV:
{

	magma_mic_dprbt_mv_param param;
//printf("je suis dans le bon cas\n");
	if(mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
		printf("scif_recv failed for DPRBT_MV\n");
		return;
    }

	call_dprbt_mv(param);

	break;
}


case magma_mic_DSWAP:
{
    magma_mic_dswap_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DSWAP\n");
        return;
    }

    call_dswap(param);

    break;
}

case magma_mic_DLASYF:
{
    magma_mic_dlasyf_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
      printf("scif_recv failed for DLASYF\n");
      return;
    }

    call_dlasyf(param);

    break;
}

case magma_mic_DTRANSPOSE:
{
    magma_mic_dtranspose_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DTRANSPOSE\n");
        return;
    }

    call_dtranspose(param);
    break;
}

case magma_mic_DLANGE:
{
    magma_mic_dlange_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DLANGE\n");
        return;
    }

    call_dlange(param);
    break;
}

case magma_mic_DLACPY:
{
    magma_mic_dlacpy_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DLACPY\n");
        return;
    }

    call_dlacpy(param);
    break;
}

case magma_mic_DAXPYCP2:
{
    magma_mic_daxpycp2_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DAXPYCP2\n");
        return;
    }

    call_daxpycp2(param);
    break;
}

case magma_mic_DSCAL:
{
    magma_mic_dscal_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DAXPYCP2\n");
        return;
    }

    dscal(&param.n, (const double *)&param.alpha, (double *)param.dx, &param.incx);
    break;
}

case magma_mic_DLASCL_DIAG:
{
    magma_mic_dlascl_diag_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DLASCL_DIAG\n");
        return;
    }

    call_dlascl_diag(param);
    break;

}

case magma_mic_IDAMAX:
{
    magma_mic_idamax_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for IDAMAX\n");
        return;
    }

    call_idamax(param);
    break;
}


// SPARSE MAGMA


case magma_mic_DJACOBIDIAGSCALE:
{
    magma_mic_djacobidiagscale_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZJACOBIDIAGSCALE\n");
        return;
    }

    call_djacobidiagscale(param);
    break;
}



#undef PRECISION_d
