/*
 *   -- micMAGMA (version 1.3.1) --
 *      Univ. of Tennessee, Knoxville
 *      Univ. of California, Berkeley
 *      Univ. of Colorado, Denver
 *      April 2012
 *
 *   @author Azzam Haidar
 *   @author Khairul Kabir
 *   @author Stan Tomov
 *   @author Hartwig Anzt
 *
 *   @generated from magma_z_kernels.h normal z -> s, Fri Jan 30 18:21:54 2015
 *
 *   This is included in magma_mic_server.cpp,
 *   inside a switch { } block in the do_work() function.
 */

#define PRECISION_s

case magma_mic_SCSRGEMV:
{
    magma_mic_scsrgemv_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SCSRGEMV\n");
        return;
    }

    mic_time = dsecnd();
    //call_scsrgemv(param);
    if (param.transa == MagmaNoTrans) {
		mkl_scsrgemv("No Transpose", &param.m, (float *)param.a, 
                     (MKL_INT *)param.ia, (MKL_INT *)param.ja, (float *)param.x, (float *)param.y);
	}else{
        mkl_scsrgemv("Transpose", &param.m, (float *)param.a,
                     (MKL_INT *)param.ia, (MKL_INT *)param.ja, (float *)param.x, (float *)param.y);
	}
	mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_SCSRGESPMM:
{
    magma_mic_scsrgespmm_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZCSRGESPMM\n");
        return;
    }

    mic_time = dsecnd();
    //call_scsrgemv(param);
    if (param.transa == MagmaNoTrans) {
        mkl_scsrmultcsr("No Transpose", &param.request, &param.sort, &param.m, &param.n, &param.k, 
                        (float *)param.a, (MKL_INT *)param.ja, (MKL_INT *)param.ia, 
                        (float *)param.b, (MKL_INT *)param.jb, (MKL_INT *)param.ib, 
                        (float *)param.c, (MKL_INT *)param.jc, (MKL_INT *)param.ic, &param.nzmax, &param.info);
	}else{
        mkl_scsrmultcsr("Transpose", &param.request, &param.sort, &param.m, &param.n, &param.k, 
                        (float *)param.a, (MKL_INT *)param.ja, (MKL_INT *)param.ia, 
                        (float *)param.b, (MKL_INT *)param.jb, (MKL_INT *)param.ib, 
                        (float *)param.c, (MKL_INT *)param.jc, (MKL_INT *)param.ic, &param.nzmax, &param.info);
	}
	mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_SDOT:
{
    //printf("sdot\n");
    magma_mic_sdot_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SDOT\n");
        return;
    }

    mic_time = dsecnd();
    #if defined (PRECISION_z ) || defined( PRECISION_c )
    sdot((float *) &param.res,
          (const MKL_INT *) &param.n, 
          (const float *) param.x, (const MKL_INT *) &param.incx,
          (const float *) param.y, (const MKL_INT *) &param.incy);
    #else
    *param.res = sdot((const MKL_INT *) &param.n,
                      (const float *) param.x, (const MKL_INT *) &param.incx,
                      (const float *) param.y, (const MKL_INT *) &param.incy);
    #endif
    mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_SNRM2:
{
    //printf("snrm2\n");
    magma_mic_snrm2_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SNRM2\n");
        return;
    }

    mic_time = dsecnd();
    *param.res = snrm2((const MKL_INT *) &param.n, (const float *) param.x, 
                        (const MKL_INT *) &param.incx);
    mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_SAXPY:
{
    //printf("saxpy\n");
    magma_mic_saxpy_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SAXPY\n");
        return;
    }

    mic_time = dsecnd();
    //call_saxpy(param);
    saxpy( (const MKL_INT *) &param.n, (const float *) &param.a, 
        (const float *) param.x,  (const MKL_INT *) &param.incx,  
        (float *) param.y, (const MKL_INT *) &param.incy );
    mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_SCOPY:
{
    //printf("scopy\n");
    magma_mic_scopy_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SCOPY\n");
        return;
    }

    mic_time = dsecnd();
    call_scopy(param);
    mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_SGEMM:
{
    //printf("sgemm\n");
    magma_mic_sgemm_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SGEMM\n");
        return;
    }

    mic_time = dsecnd();
    call_sgemm(param);
    mic_time = dsecnd() - mic_time;

    m = param.m;
    n = param.n;
    k = param.k;
    gflops = MICFLOPS_GEMM((float)m, (float)n, (float)k) * 1e-9;
    //printf("sgemm: %d\t%d\t%d\t%8.2lf\t%8.2lf\t%8.2lf\n", m, n, k, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_SSYMM:
{
    //printf("ssymm\n");
    magma_mic_ssymm_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SSYMM\n");
        return;
    }

    mic_time = dsecnd();
    call_ssymm(param);
    mic_time = dsecnd() - mic_time;

    m = param.m;
    n = param.n;
    magma_side_t side = param.side;

    //gflops = MICFLOPS_SYMM((magma_side_t)side, (float)m, (float)n ) * 1e-9;
    //printf("ssymm: %d\t%d\t%g\t%g\t%g\n", m, n, mic_time,gflops, gflops/mic_time);
    break;
}

case magma_mic_SGEMV:
{
    //printf("sgemv\n");
    magma_mic_sgemv_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SGEMV\n");
        return;
    }

    mic_time = dsecnd();
    call_sgemv(param);
    mic_time = dsecnd() - mic_time;

    m = param.m;
    n = param.n;
    //gflops = MICFLOPS_GEMV((float)m, (float)n) * 1e-9;
    //printf("sgemv: %d\t%d\t%g\t%g\t%g\n", m, n, mic_time,
    //        gflops, gflops/mic_time);
    break;
}

case magma_mic_SSYMV:
{
    //printf("ssymv\n");
    magma_mic_ssymv_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SSYMV\n");
        return;
    }

    mic_time = dsecnd();
    call_ssymv(param);
    mic_time = dsecnd() - mic_time;

    break;
}

case magma_mic_SSYMV2:
{
    //printf("ssymv2\n");
    magma_mic_ssymv2_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SSYMV\n");
        return;
    }

    mic_time = dsecnd();
    call_ssymv2(param);
    mic_time = dsecnd() - mic_time;

    break;
}

case magma_mic_STRSM:
{
    //printf("strsm\n");
    magma_mic_strsm_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for STRSM\n");
        return;
    }

    mic_time = dsecnd();
    call_strsm(param);
    mic_time = dsecnd() - mic_time;

    side = param.side;
    m = param.m;
    n = param.n;

    if (side == MagmaLeft)
        gflops = MICFLOPS_TRSM_L((float)m, (float)n) * 1e-9;
    else
        gflops = MICFLOPS_TRSM_R((float)m, (float)n) * 1e-9;

    //printf("strsm: %c\t%d\t%d\t%f\t%f\t%f\n", side, m, n, mic_time,
    //     gflops, gflops/mic_time);
    break;
}

case magma_mic_STRSV:
{
    //printf("strsv\n");

    magma_mic_strsv_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for STRSV\n");
        return;
    }
    
    mic_time = dsecnd();
    call_strsv(param);
    mic_time = dsecnd() - mic_time;
    
    break;
}

case magma_mic_STRMV:
{
    //printf("strmv\n");
    
    magma_mic_strmv_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for STRMV\n");
        return;
    }
    
    mic_time = dsecnd();
    call_strmv(param);
    mic_time = dsecnd() - mic_time;
    
    break;
}

case magma_mic_STRMM:
{
    //printf("strmm\n");
    magma_mic_strmm_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for STRMM\n");
        return;
    }

    mic_time = dsecnd();
    call_strmm(param);
    mic_time = dsecnd() - mic_time;

    side = param.side;
    m = param.m;
    n = param.n;

    //if (side == MagmaLeft)
    //    gflops = MICFLOPS_TRSM_L((float)m, (float)n) * 1e-9;
    //else
    //    gflops = MICFLOPS_TRSM_R((float)m, (float)n) * 1e-9;

    //printf("strmm: %c\t%d\t%d\t%f\t%f\t%f\n", side, m, n, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_SSYRK:
{
    //printf("ssyrk\n");
    magma_mic_ssyrk_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SSYRK\n");
        return;
    }

    mic_time = dsecnd();
    call_ssyrk(param);
    mic_time = dsecnd() - mic_time;

    n = param.n;
    k = param.k;
    //gflops = MICFLOPS_SYRK((float)n, (float)k) * 1e-9;
    //printf("ssyrk: %d\t%d\t%f\t%f\t%f\n", n, k, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_SSYRK_MMIC:
{
    //printf("ssyrk\n");
    magma_mic_ssyrk_mmic_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SSYRK_MMIC\n");
        return;
    }

    mic_time = dsecnd();
    call_ssyrk_mmic(param);
    mic_time = dsecnd() - mic_time;

    n = param.n;
    k = param.k;
    //gflops = MICFLOPS_SYRK((float)n, (float)k) * 1e-9;
    //printf("ssyrk: %d\t%d\t%f\t%f\t%f\n", n, k, mic_time, gflops, gflops/mic_time);
    break;
}


case magma_mic_SSYR2K:
{
    //printf("ssyr2k\n");
    magma_mic_ssyr2k_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SSYR2K\n");
        return;
    }

    mic_time = dsecnd();
    call_ssyr2k(param);
    mic_time = dsecnd() - mic_time;

    n = param.n;
    k = param.k;
    //gflops = MICFLOPS_SYR2K((float)n, (float)k) * 1e-9;
    //printf("ssyr2k: %d\t%d\t%f\t%f\t%f\n", n, k, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_SSYR2K_MMIC:
{
    //printf("ssyr2k_mmic\n");
    magma_mic_ssyr2k_mmic_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SSYR2K_MMIC\n");
        return;
    }

    //mic_time = dsecnd();
    call_ssyr2k_mmic(param);
    //mic_time = dsecnd() - mic_time;

    n = param.n;
    k = param.k;
    //gflops = MICFLOPS_SYRK((float)n, (float)k) * 1e-9;
    //printf("ssyrk: %d\t%d\t%f\t%f\t%f\n", n, k, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_SLARFT:
{
    //printf("slarft\n");
    magma_mic_slarft_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SLARFT\n");
        return;
    }

    //mic_time = dsecnd();
    call_slarft(param);
    //mic_time = dsecnd() - mic_time;

    //gflops = MICFLOPS_GEMM((float)m, (float)n, (float)k) * 1e-9;
    //printf("slarft: %d\t%d\t%d\t%f\t%f\t%f\n", m, n, k, mic_time,
    //     gflops, gflops/mic_time);
    break;
}

case magma_mic_SLARFB:
{
    //printf("slarfb\n");
    magma_mic_slarfb_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SLARFB\n");
        return;
    }

    //mic_time = dsecnd();
    call_slarfb(param);
    //mic_time = dsecnd() - mic_time;

    //gflops = MICFLOPS_GEMM((float)m, (float)n, (float)k) * 1e-9;
    //printf("slarfb: %d\t%d\t%d\t%f\t%f\t%f\n", m, n, k, mic_time,
    //     gflops, gflops/mic_time);
    break;
}

case magma_mic_SLARFB_GEMM:
{
    //printf("slarfb\n");
    magma_mic_slarfb_gemm_param slarfb_gemm_param;
    if (mic_recv(&slarfb_gemm_param, sizeof(slarfb_gemm_param)) == MIC_FAILURE) {
        printf("scif_recv failed for SLARFB_GEMM\n");
        return;
    }

    //mic_time = dsecnd();
    call_slarfb_gemm(slarfb_gemm_param);
    //mic_time = dsecnd() - mic_time;

    //gflops = MICFLOPS_GEMM((float)m, (float)n, (float)k) * 1e-9;
    //printf("slarfb: %d\t%d\t%d\t%f\t%f\t%f\n", m, n, k, mic_time,
    //     gflops, gflops/mic_time);
    break;
}

case magma_mic_STRSM_GEMM:
{
    //printf("strsmgemm\n");
    magma_mic_strsmgemm_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for STRSM_GEMM\n");
        return;
    }

    mic_time = dsecnd();
    call_strsmgemm(param);
    mic_time = dsecnd() - mic_time;

    break;
}

case magma_mic_SLASET:
{
    magma_mic_slaset_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SLASET\n");
        return;
    }

    call_slaset(param);

    break;
}

case magma_mic_SLASWP:
{
    magma_mic_slaswp_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SLASWP\n");
        return;
    }

    call_slaswp(param);

    break;
}

case magma_mic_SPRBT:
{

	magma_mic_sprbt_param param;
//printf("je suis dans le bon cas\n");
	if(mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
		printf("scif_recv failed for SPRBT\n");
		return;
    }

	call_sprbt(param);

	break;
}

case magma_mic_SPRBT_MTV:
{

	magma_mic_sprbt_mtv_param param;
//printf("je suis dans le bon cas\n");
	if(mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
		printf("scif_recv failed for SPRBT_MTV\n");
		return;
    }

	call_sprbt_mtv(param);

	break;
}

case magma_mic_SPRBT_MV:
{

	magma_mic_sprbt_mv_param param;
//printf("je suis dans le bon cas\n");
	if(mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
		printf("scif_recv failed for SPRBT_MV\n");
		return;
    }

	call_sprbt_mv(param);

	break;
}


case magma_mic_SSWAP:
{
    magma_mic_sswap_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SSWAP\n");
        return;
    }

    call_sswap(param);

    break;
}

case magma_mic_SLASYF:
{
    magma_mic_slasyf_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
      printf("scif_recv failed for SLASYF\n");
      return;
    }

    call_slasyf(param);

    break;
}

case magma_mic_STRANSPOSE:
{
    magma_mic_stranspose_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for STRANSPOSE\n");
        return;
    }

    call_stranspose(param);
    break;
}

case magma_mic_SLANGE:
{
    magma_mic_slange_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SLANGE\n");
        return;
    }

    call_slange(param);
    break;
}

case magma_mic_SLACPY:
{
    magma_mic_slacpy_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SLACPY\n");
        return;
    }

    call_slacpy(param);
    break;
}

case magma_mic_SAXPYCP2:
{
    magma_mic_saxpycp2_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SAXPYCP2\n");
        return;
    }

    call_saxpycp2(param);
    break;
}

case magma_mic_SSCAL:
{
    magma_mic_sscal_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SAXPYCP2\n");
        return;
    }

    sscal(&param.n, (const float *)&param.alpha, (float *)param.dx, &param.incx);
    break;
}

case magma_mic_SLASCL_DIAG:
{
    magma_mic_slascl_diag_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for SLASCL_DIAG\n");
        return;
    }

    call_slascl_diag(param);
    break;

}

case magma_mic_ISAMAX:
{
    magma_mic_isamax_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ISAMAX\n");
        return;
    }

    call_isamax(param);
    break;
}


// SPARSE MAGMA


case magma_mic_SJACOBIDIAGSCALE:
{
    magma_mic_sjacobidiagscale_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZJACOBIDIAGSCALE\n");
        return;
    }

    call_sjacobidiagscale(param);
    break;
}



#undef PRECISION_s
