/*
 *   -- micMAGMA (version 1.3.1) --
 *      Univ. of Tennessee, Knoxville
 *      Univ. of California, Berkeley
 *      Univ. of Colorado, Denver
 *      April 2012
 *
 *   @author Azzam Haidar
 *   @author Khairul Kabir
 *   @author Stan Tomov
 *   @author Hartwig Anzt
 *
 *   @precisions normal z -> s d c
 *
 *   This is included in magma_mic_server.cpp,
 *   inside a switch { } block in the do_work() function.
 */

#define PRECISION_z

case magma_mic_ZCSRGEMV:
{
    magma_mic_zcsrgemv_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZCSRGEMV\n");
        return;
    }

    mic_time = dsecnd();
    //call_zcsrgemv(param);
    if (param.transa == MagmaNoTrans) {
		mkl_zcsrgemv("No Transpose", &param.m, (MKL_Complex16 *)param.a, 
                     (MKL_INT *)param.ia, (MKL_INT *)param.ja, (MKL_Complex16 *)param.x, (MKL_Complex16 *)param.y);
	}else{
        mkl_zcsrgemv("Transpose", &param.m, (MKL_Complex16 *)param.a,
                     (MKL_INT *)param.ia, (MKL_INT *)param.ja, (MKL_Complex16 *)param.x, (MKL_Complex16 *)param.y);
	}
	mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_ZCSRGESPMM:
{
    magma_mic_zcsrgespmm_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZCSRGESPMM\n");
        return;
    }

    mic_time = dsecnd();
    //call_zcsrgemv(param);
    if (param.transa == MagmaNoTrans) {
        mkl_zcsrmultcsr("No Transpose", &param.request, &param.sort, &param.m, &param.n, &param.k, 
                        (MKL_Complex16 *)param.a, (MKL_INT *)param.ja, (MKL_INT *)param.ia, 
                        (MKL_Complex16 *)param.b, (MKL_INT *)param.jb, (MKL_INT *)param.ib, 
                        (MKL_Complex16 *)param.c, (MKL_INT *)param.jc, (MKL_INT *)param.ic, &param.nzmax, &param.info);
	}else{
        mkl_zcsrmultcsr("Transpose", &param.request, &param.sort, &param.m, &param.n, &param.k, 
                        (MKL_Complex16 *)param.a, (MKL_INT *)param.ja, (MKL_INT *)param.ia, 
                        (MKL_Complex16 *)param.b, (MKL_INT *)param.jb, (MKL_INT *)param.ib, 
                        (MKL_Complex16 *)param.c, (MKL_INT *)param.jc, (MKL_INT *)param.ic, &param.nzmax, &param.info);
	}
	mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_ZDOTC:
{
    //printf("zdotc\n");
    magma_mic_zdotc_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZDOTC\n");
        return;
    }

    mic_time = dsecnd();
    #if defined (PRECISION_z ) || defined( PRECISION_c )
    zdotc((MKL_Complex16 *) &param.res,
          (const MKL_INT *) &param.n, 
          (const MKL_Complex16 *) param.x, (const MKL_INT *) &param.incx,
          (const MKL_Complex16 *) param.y, (const MKL_INT *) &param.incy);
    #else
    *param.res = zdotc((const MKL_INT *) &param.n,
                      (const MKL_Complex16 *) param.x, (const MKL_INT *) &param.incx,
                      (const MKL_Complex16 *) param.y, (const MKL_INT *) &param.incy);
    #endif
    mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_DZNRM2:
{
    //printf("dznrm2\n");
    magma_mic_dznrm2_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for DZNRM2\n");
        return;
    }

    mic_time = dsecnd();
    *param.res = dznrm2((const MKL_INT *) &param.n, (const MKL_Complex16 *) param.x, 
                        (const MKL_INT *) &param.incx);
    mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_ZAXPY:
{
    //printf("zaxpy\n");
    magma_mic_zaxpy_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZAXPY\n");
        return;
    }

    mic_time = dsecnd();
    //call_zaxpy(param);
    zaxpy( (const MKL_INT *) &param.n, (const MKL_Complex16 *) &param.a, 
        (const MKL_Complex16 *) param.x,  (const MKL_INT *) &param.incx,  
        (MKL_Complex16 *) param.y, (const MKL_INT *) &param.incy );
    mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_ZCOPY:
{
    //printf("zcopy\n");
    magma_mic_zcopy_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZCOPY\n");
        return;
    }

    mic_time = dsecnd();
    call_zcopy(param);
    mic_time = dsecnd() - mic_time;

    break;
}


case magma_mic_ZGEMM:
{
    //printf("zgemm\n");
    magma_mic_zgemm_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZGEMM\n");
        return;
    }

    mic_time = dsecnd();
    call_zgemm(param);
    mic_time = dsecnd() - mic_time;

    m = param.m;
    n = param.n;
    k = param.k;
    gflops = MICFLOPS_GEMM((double)m, (double)n, (double)k) * 1e-9;
    //printf("zgemm: %d\t%d\t%d\t%8.2lf\t%8.2lf\t%8.2lf\n", m, n, k, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_ZHEMM:
{
    //printf("zhemm\n");
    magma_mic_zhemm_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZHEMM\n");
        return;
    }

    mic_time = dsecnd();
    call_zhemm(param);
    mic_time = dsecnd() - mic_time;

    m = param.m;
    n = param.n;
    magma_side_t side = param.side;

    //gflops = MICFLOPS_SYMM((magma_side_t)side, (double)m, (double)n ) * 1e-9;
    //printf("zhemm: %d\t%d\t%g\t%g\t%g\n", m, n, mic_time,gflops, gflops/mic_time);
    break;
}

case magma_mic_ZGEMV:
{
    //printf("zgemv\n");
    magma_mic_zgemv_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZGEMV\n");
        return;
    }

    mic_time = dsecnd();
    call_zgemv(param);
    mic_time = dsecnd() - mic_time;

    m = param.m;
    n = param.n;
    //gflops = MICFLOPS_GEMV((double)m, (double)n) * 1e-9;
    //printf("zgemv: %d\t%d\t%g\t%g\t%g\n", m, n, mic_time,
    //        gflops, gflops/mic_time);
    break;
}

case magma_mic_ZHEMV:
{
    //printf("zhemv\n");
    magma_mic_zhemv_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZHEMV\n");
        return;
    }

    mic_time = dsecnd();
    call_zhemv(param);
    mic_time = dsecnd() - mic_time;

    break;
}

case magma_mic_ZSYMV2:
{
    //printf("zsymv2\n");
    magma_mic_zsymv2_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZHEMV\n");
        return;
    }

    mic_time = dsecnd();
    call_zsymv2(param);
    mic_time = dsecnd() - mic_time;

    break;
}

case magma_mic_ZTRSM:
{
    //printf("ztrsm\n");
    magma_mic_ztrsm_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZTRSM\n");
        return;
    }

    mic_time = dsecnd();
    call_ztrsm(param);
    mic_time = dsecnd() - mic_time;

    side = param.side;
    m = param.m;
    n = param.n;

    if (side == MagmaLeft)
        gflops = MICFLOPS_TRSM_L((double)m, (double)n) * 1e-9;
    else
        gflops = MICFLOPS_TRSM_R((double)m, (double)n) * 1e-9;

    //printf("ztrsm: %c\t%d\t%d\t%f\t%f\t%f\n", side, m, n, mic_time,
    //     gflops, gflops/mic_time);
    break;
}

case magma_mic_ZTRSV:
{
    //printf("ztrsv\n");

    magma_mic_ztrsv_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZTRSV\n");
        return;
    }
    
    mic_time = dsecnd();
    call_ztrsv(param);
    mic_time = dsecnd() - mic_time;
    
    break;
}

case magma_mic_ZTRMV:
{
    //printf("ztrmv\n");
    
    magma_mic_ztrmv_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZTRMV\n");
        return;
    }
    
    mic_time = dsecnd();
    call_ztrmv(param);
    mic_time = dsecnd() - mic_time;
    
    break;
}

case magma_mic_ZTRMM:
{
    //printf("ztrmm\n");
    magma_mic_ztrmm_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZTRMM\n");
        return;
    }

    mic_time = dsecnd();
    call_ztrmm(param);
    mic_time = dsecnd() - mic_time;

    side = param.side;
    m = param.m;
    n = param.n;

    //if (side == MagmaLeft)
    //    gflops = MICFLOPS_TRSM_L((float)m, (float)n) * 1e-9;
    //else
    //    gflops = MICFLOPS_TRSM_R((float)m, (float)n) * 1e-9;

    //printf("ztrmm: %c\t%d\t%d\t%f\t%f\t%f\n", side, m, n, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_ZHERK:
{
    //printf("zherk\n");
    magma_mic_zherk_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZHERK\n");
        return;
    }

    mic_time = dsecnd();
    call_zherk(param);
    mic_time = dsecnd() - mic_time;

    n = param.n;
    k = param.k;
    //gflops = MICFLOPS_SYRK((float)n, (float)k) * 1e-9;
    //printf("zherk: %d\t%d\t%f\t%f\t%f\n", n, k, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_ZHERK_MMIC:
{
    //printf("zherk\n");
    magma_mic_zherk_mmic_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZHERK_MMIC\n");
        return;
    }

    mic_time = dsecnd();
    call_zherk_mmic(param);
    mic_time = dsecnd() - mic_time;

    n = param.n;
    k = param.k;
    //gflops = MICFLOPS_SYRK((float)n, (float)k) * 1e-9;
    //printf("zherk: %d\t%d\t%f\t%f\t%f\n", n, k, mic_time, gflops, gflops/mic_time);
    break;
}


case magma_mic_ZHER2K:
{
    //printf("zher2k\n");
    magma_mic_zher2k_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZHER2K\n");
        return;
    }

    mic_time = dsecnd();
    call_zher2k(param);
    mic_time = dsecnd() - mic_time;

    n = param.n;
    k = param.k;
    //gflops = MICFLOPS_SYR2K((float)n, (float)k) * 1e-9;
    //printf("zher2k: %d\t%d\t%f\t%f\t%f\n", n, k, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_ZHER2K_MMIC:
{
    //printf("zher2k_mmic\n");
    magma_mic_zher2k_mmic_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZHER2K_MMIC\n");
        return;
    }

    //mic_time = dsecnd();
    call_zher2k_mmic(param);
    //mic_time = dsecnd() - mic_time;

    n = param.n;
    k = param.k;
    //gflops = MICFLOPS_SYRK((float)n, (float)k) * 1e-9;
    //printf("zherk: %d\t%d\t%f\t%f\t%f\n", n, k, mic_time, gflops, gflops/mic_time);
    break;
}

case magma_mic_ZLARFT:
{
    //printf("zlarft\n");
    magma_mic_zlarft_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZLARFT\n");
        return;
    }

    //mic_time = dsecnd();
    call_zlarft(param);
    //mic_time = dsecnd() - mic_time;

    //gflops = MICFLOPS_GEMM((float)m, (float)n, (float)k) * 1e-9;
    //printf("zlarft: %d\t%d\t%d\t%f\t%f\t%f\n", m, n, k, mic_time,
    //     gflops, gflops/mic_time);
    break;
}

case magma_mic_ZLARFB:
{
    //printf("zlarfb\n");
    magma_mic_zlarfb_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZLARFB\n");
        return;
    }

    //mic_time = dsecnd();
    call_zlarfb(param);
    //mic_time = dsecnd() - mic_time;

    //gflops = MICFLOPS_GEMM((float)m, (float)n, (float)k) * 1e-9;
    //printf("zlarfb: %d\t%d\t%d\t%f\t%f\t%f\n", m, n, k, mic_time,
    //     gflops, gflops/mic_time);
    break;
}

case magma_mic_ZLARFB_GEMM:
{
    //printf("zlarfb\n");
    magma_mic_zlarfb_gemm_param zlarfb_gemm_param;
    if (mic_recv(&zlarfb_gemm_param, sizeof(zlarfb_gemm_param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZLARFB_GEMM\n");
        return;
    }

    //mic_time = dsecnd();
    call_zlarfb_gemm(zlarfb_gemm_param);
    //mic_time = dsecnd() - mic_time;

    //gflops = MICFLOPS_GEMM((float)m, (float)n, (float)k) * 1e-9;
    //printf("zlarfb: %d\t%d\t%d\t%f\t%f\t%f\n", m, n, k, mic_time,
    //     gflops, gflops/mic_time);
    break;
}

case magma_mic_ZTRSM_GEMM:
{
    //printf("ztrsmgemm\n");
    magma_mic_ztrsmgemm_param param;

    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZTRSM_GEMM\n");
        return;
    }

    mic_time = dsecnd();
    call_ztrsmgemm(param);
    mic_time = dsecnd() - mic_time;

    break;
}

case magma_mic_ZLASET:
{
    magma_mic_zlaset_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZLASET\n");
        return;
    }

    call_zlaset(param);

    break;
}

case magma_mic_ZLASWP:
{
    magma_mic_zlaswp_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZLASWP\n");
        return;
    }

    call_zlaswp(param);

    break;
}

case magma_mic_ZPRBT:
{

	magma_mic_zprbt_param param;
//printf("je suis dans le bon cas\n");
	if(mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
		printf("scif_recv failed for ZPRBT\n");
		return;
    }

	call_zprbt(param);

	break;
}

case magma_mic_ZPRBT_MTV:
{

	magma_mic_zprbt_mtv_param param;
//printf("je suis dans le bon cas\n");
	if(mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
		printf("scif_recv failed for ZPRBT_MTV\n");
		return;
    }

	call_zprbt_mtv(param);

	break;
}

case magma_mic_ZPRBT_MV:
{

	magma_mic_zprbt_mv_param param;
//printf("je suis dans le bon cas\n");
	if(mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
		printf("scif_recv failed for ZPRBT_MV\n");
		return;
    }

	call_zprbt_mv(param);

	break;
}


case magma_mic_ZSWAP:
{
    magma_mic_zswap_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZSWAP\n");
        return;
    }

    call_zswap(param);

    break;
}

case magma_mic_ZLAHEF:
{
    magma_mic_zlahef_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
      printf("scif_recv failed for ZLAHEF\n");
      return;
    }

    call_zlahef(param);

    break;
}

case magma_mic_ZTRANSPOSE:
{
    magma_mic_ztranspose_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZTRANSPOSE\n");
        return;
    }

    call_ztranspose(param);
    break;
}

case magma_mic_ZLANGE:
{
    magma_mic_zlange_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZLANGE\n");
        return;
    }

    call_zlange(param);
    break;
}

case magma_mic_ZLACPY:
{
    magma_mic_zlacpy_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZLACPY\n");
        return;
    }

    call_zlacpy(param);
    break;
}

case magma_mic_ZAXPYCP2:
{
    magma_mic_zaxpycp2_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZAXPYCP2\n");
        return;
    }

    call_zaxpycp2(param);
    break;
}

case magma_mic_ZSCAL:
{
    magma_mic_zscal_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZAXPYCP2\n");
        return;
    }

    zscal(&param.n, (const MKL_Complex16 *)&param.alpha, (MKL_Complex16 *)param.dx, &param.incx);
    break;
}

case magma_mic_ZLASCL_DIAG:
{
    magma_mic_zlascl_diag_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZLASCL_DIAG\n");
        return;
    }

    call_zlascl_diag(param);
    break;

}

case magma_mic_IZAMAX:
{
    magma_mic_izamax_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for IZAMAX\n");
        return;
    }

    call_izamax(param);
    break;
}


// SPARSE MAGMA


case magma_mic_ZJACOBIDIAGSCALE:
{
    magma_mic_zjacobidiagscale_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZJACOBIDIAGSCALE\n");
        return;
    }

    call_zjacobidiagscale(param);
    break;
}



#undef PRECISION_z
