/*
    -- MAGMA (version 1.3.1) --
       Univ. of Tennessee, Knoxville
       Univ. of California, Berkeley
       Univ. of Colorado, Denver
       @date January 2015

       @generated from zhemv_magma_cpu.cpp normal z -> d, Fri Jan 30 18:21:52 2015
*/


#include <stdlib.h>
#include <string.h>
//#include <mkl.h>
#include <omp.h>

#include "common_magma.h"


void magmablas_dsymv_work_cpu(magma_uplo_t uplo, magma_int_t *pN, double *palpha, double *A, magma_int_t *plda, double *X, magma_int_t *pincx, double *pbeta, double *Y, magma_int_t *pincy, double *y) 
{
	#define BS 128
        #define A(i,j) A + (i) + (j)*lda
        #define y(i,j) y + (i) + (j)*N
        #define Y(i)   Y + (i)
        #define X(i)   X + (i)

	magma_int_t N   = *pN;
        magma_int_t lda = *plda;	
	magma_int_t incx = *pincx;
	magma_int_t incy = *pincy;
	double alpha = *palpha;
	double beta  = *pbeta;

	magma_int_t nthr;

	#pragma omp parallel 
	{
		nthr = omp_get_num_threads();
	}

	#pragma omp parallel num_threads(nthr)
	{
		magma_int_t id = omp_get_thread_num();
		memset( y(0,id), 0, N*sizeof(double) );
	}

        blasf77_dscal(&N, &beta, Y, &incy);

	#pragma omp parallel num_threads(nthr)
	{
		magma_int_t id = omp_get_thread_num();
		memset( y(0,id), 0, N * sizeof(double) );

		double beta_one  = MAGMA_D_MAKE( 1.0,  0. );
		for(magma_int_t i = id*BS; i < N; i+= nthr * BS){
			magma_int_t n = min(N - i, BS);

			// compute diagonal first
			blasf77_dsymv( lapack_uplo_const(uplo), &n, &alpha,
					A(i,i), &lda, X(i), &incx,
					&beta_one, y(i, id), &incy );
			
			// Compute the rest of the column
			for(magma_int_t j = i+n; j< N; j+= BS){
				magma_int_t nn = min(N - j, BS);
				blasf77_dgemv(MagmaNoTransStr, &nn, &n, &alpha,
						A(j,i), &lda, X(i), &incx,
						&beta_one, y(j, id), &incy );
				blasf77_dgemv(MagmaConjTransStr, &nn, &n, &alpha,
						A(j,i), &lda, X(j), &incx,
						&beta_one, y(i, id), &incy );
			}
		}

		// Barier before we add all numbers in the result
		#pragma omp barrier

		magma_int_t block  = ( N + (nthr - 1))/nthr;
		magma_int_t start  = id * block;
		magma_int_t  n      = block;
		if (id == (nthr - 1)){
			n = N - start;
		}

		for(magma_int_t i = 0; i < nthr; i++){
			blasf77_daxpy(&n, &beta_one, y(start, i), &incy, Y(start), &incy);
		}
	}
}

void magmablas_dsymv_cpu(magma_uplo_t uplo, magma_int_t *pN, double *palpha, double *A, magma_int_t *plda, double *X, magma_int_t *pincx, double *pbeta, double *Y, magma_int_t *pincy) 
{
	double *y;
	magma_int_t nthr;
	magma_int_t N   = *pN;

	#pragma omp parallel 
	{
		nthr = omp_get_num_threads();
	}
	//default number of threads is 32; so we need to set 16 using export OMP_NUM_THREADS=16;
	//printf("nthreads1: %d\n", nthr);
	posix_memalign((void **)&y,   4096, sizeof(double) * N * nthr);
	magmablas_dsymv_work_cpu(uplo, pN, palpha, A, plda, X, pincx, pbeta, Y, pincy, y); 
	free(y);
}


