/*
 *  -- MAGMA (version 1.3.1) --
 *     Univ. of Tennessee, Knoxville
 *     Univ. of California, Berkeley
 *     Univ. of Colorado, Denver
 *     @date January 2015
 *
 *  @generated from testing_zhemv.cpp normal z -> c, Fri Jan 30 18:21:55 2015
 *
 **/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <omp.h>

#include "flops.h"
#include "magma.h"
#include "magma_lapack.h"
#include "testings.h"

int main(int argc, char** argv)
{
    /* Initialize */
    magma_queue_t  queue;
    magma_device_t device;
    magma_int_t num = 0;
    magma_int_t err;
    magma_init();
    err = magma_get_devices( &device, 1, &num );
    if ( err != MAGMA_SUCCESS or num < 1 ) {
      fprintf( stderr, "magma_get_devices failed: %d\n", (int) err );
      exit(-1);
    }
    err = magma_queue_create( device, &queue );
    if ( err != MAGMA_SUCCESS ) {
      fprintf( stderr, "magma_queue_create failed: %d\n", (int) err );
      exit(-1);
    }

    real_Double_t   gflops, mic_perf, mic_time, cpu_perf, cpu_time;
    float          mic_error, work[1];
    magma_int_t ione     = 1;
    magma_int_t ISEED[4] = {0,0,0,1};
    magma_int_t N, lda, sizeA, sizeX, sizeY, blocks, ldwork;
    magma_int_t incx = 1;
    magma_int_t incy = 1;
    magma_int_t nb   = 64;
    magmaFloatComplex c_neg_one = MAGMA_C_NEG_ONE;
    magmaFloatComplex alpha = MAGMA_C_MAKE(  1.5, -2.3 );
    magmaFloatComplex beta  = MAGMA_C_MAKE( -0.6,  0.8 );
    magmaFloatComplex *A, *X, *Y, *Ymic, *Ya, *Yna;
    magmaFloatComplex *dA, *dX, *dY, *dC_work;
    
    magma_opts opts;
    parse_opts( argc, argv, &opts );

    printf("    N   MIC Gflop/s (ms)   CPU MKL Gflop/s (ms)  MAGMA_CPU Gflop/s (ms) MAGMA_CPU_NA gflop/s (ms) MIC error    MAGMA_error    MAGMA_error2\n");
    printf("==========================================================================================================================================\n");
    for( int i = 0; i < opts.ntest; ++i ) {
        for( int iter = 0; iter < opts.niter; ++iter ) {
            N = opts.nsize[i];
            lda    = ((N + 31)/32)*32;
            sizeA  = N*lda;
            sizeX  = N*incx;
            sizeY  = N*incy;
            gflops = FLOPS_CHEMV( N ) / 1e9;
            
            TESTING_MALLOC_HOST( A,    magmaFloatComplex, sizeA );
            TESTING_MALLOC_HOST( X,    magmaFloatComplex, sizeX );
            TESTING_MALLOC_HOST( Y,    magmaFloatComplex, sizeY );
            TESTING_MALLOC_HOST( Ya, magmaFloatComplex, sizeY );
            TESTING_MALLOC_HOST( Yna, magmaFloatComplex, sizeY );
            TESTING_MALLOC_HOST( Ymic, magmaFloatComplex, sizeY );
            
            TESTING_MALLOC_DEV( dA, magmaFloatComplex, sizeA );
            TESTING_MALLOC_DEV( dX, magmaFloatComplex, sizeX );
            TESTING_MALLOC_DEV( dY, magmaFloatComplex, sizeY );
            
            blocks = (N + nb - 1) / nb;
            ldwork = lda * (blocks + 1);
            TESTING_MALLOC_DEV( dC_work, magmaFloatComplex, ldwork );
            
            /* Initialize the matrix */
            lapackf77_clarnv( &ione, ISEED, &sizeA, A );
            magma_cmake_hermitian( N, A, lda );
            lapackf77_clarnv( &ione, ISEED, &sizeX, X );
            lapackf77_clarnv( &ione, ISEED, &sizeY, Y );
	    lapackf77_clacpy(MagmaUpperLowerStr, &N, &ione, Y, &ione, Ya, &ione );
	    lapackf77_clacpy(MagmaUpperLowerStr, &N, &ione, Y, &ione, Yna, &ione );
            
            /* =====================================================================
               Performs operation using MIC
               =================================================================== */
            magma_csetmatrix( N, N, A, 0, lda, dA, 0, lda, queue );
            //magma_csetvector( N, X, incx, dX, incx );
            //magma_csetvector( N, Y, incy, dY, incy );
            magma_csetmatrix( N, 1, X, 0, N, dX, 0, N, queue );
            magma_csetmatrix( N, 1, Y, 0, N, dY, 0, N, queue );
            
            magma_chemv( opts.uplo, N, alpha, dA, 0, lda, dX, 0, incx, beta, dY, 0, incy, queue );
            //magma_csetvector( N, Y, incy, dY, incy );
            magma_csetmatrix( N, 1, Y, 0, N, dY, 0, N, queue );

            mic_time = magma_sync_wtime( 0 );
            magma_chemv( opts.uplo, N, alpha, dA, 0, lda, dX, 0, incx, beta, dY, 0, incy, queue );
            mic_time = magma_sync_wtime( 0 ) - mic_time;
            mic_perf = gflops / mic_time;
           
            if (opts.check) {
                //magma_cgetvector( N, dY, incy, Ymic, incy );
                magma_cgetmatrix( N, 1, dY, 0, N, Ymic, 0, N, queue );

                /* =====================================================================
                   Performs operation using CPU BLAS
                   =================================================================== */
                cpu_time = magma_wtime();
                blasf77_chemv( lapack_const(opts.uplo), &N, &alpha, A, &lda, X, &incx, &beta, Y, &incy );
                cpu_time = magma_wtime() - cpu_time;
                cpu_perf = gflops / cpu_time;
		
		real_Double_t magma_cpu_time = magma_wtime();
                magmablas_chemv_cpu(opts.uplo, &N, &alpha, A, &lda, X, &incx, &beta, Ya, &incy );
                magma_cpu_time = magma_wtime() - magma_cpu_time;
                real_Double_t magma_cpu_perf = gflops / magma_cpu_time;

		magmaFloatComplex *symvwork;
		magma_int_t nthr;
		#pragma omp parallel 
		{
			nthr = omp_get_num_threads();
		}

		TESTING_MALLOC_HOST( symvwork,    magmaFloatComplex, N * nthr);
		real_Double_t magma_cpu_time_na = magma_wtime();
                magmablas_chemv_work_cpu(opts.uplo, &N, &alpha, A, &lda, X, &incx, &beta, Yna, &incy, symvwork );
                magma_cpu_time_na = magma_wtime() - magma_cpu_time_na;
                real_Double_t magma_cpu_perf_na = gflops / magma_cpu_time_na;
		TESTING_FREE_HOST( symvwork );

                /* =====================================================================
                   Check the result
                   =================================================================== */
                blasf77_caxpy( &N, &c_neg_one, Y, &incy, Ymic, &incy );
                mic_error = lapackf77_clange( "M", &N, &ione, Ymic, &N, work ) / N;
                
		blasf77_caxpy( &N, &c_neg_one, Y, &incy, Ya, &incy );
                real_Double_t cpu_errora = lapackf77_clange( "M", &N, &ione, Ya, &N, work ) / N;

		blasf77_caxpy( &N, &c_neg_one, Y, &incy, Yna, &incy );
                real_Double_t cpu_errorna = lapackf77_clange( "M", &N, &ione, Yna, &N, work ) / N;
                
		printf("%5d   %7.2f (%7.2f)    %7.2f (%7.2f)   %7.2f (%7.2f)    %7.2f (%7.2f) \t%8.2e  \t%8.2e  \t%8.2e\n",
                        (int) N,
                        mic_perf, 1000.*mic_time,
                        cpu_perf,    1000.*cpu_time, magma_cpu_perf, 1000. * magma_cpu_time, magma_cpu_perf_na, 1000. * magma_cpu_time_na,
                        mic_error, cpu_errora, cpu_errorna );
            } else {
                printf("%5d   %7.2f (%7.2f)    --- (---)   ---\n",
                        (int) N,
                        mic_perf, 1000.*mic_time);
            }
            
            TESTING_FREE_HOST( A );
            TESTING_FREE_HOST( X );
            TESTING_FREE_HOST( Y );
            TESTING_FREE_HOST( Ya );
            TESTING_FREE_HOST( Yna );
            TESTING_FREE_HOST( Ymic );

            TESTING_FREE_DEV( dA );
            TESTING_FREE_DEV( dX );
            TESTING_FREE_DEV( dY );
            TESTING_FREE_DEV( dC_work );
        }
        if ( opts.niter > 1 ) {
            printf( "\n" );
        }
    }

    magma_queue_destroy( queue );
    magma_finalize();

    return 0;
}
